#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
THOR detects differential peaks in multiple ChIP-seq profiles associated
with two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhoff (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

from __future__ import print_function
from mpmath import gamma, rf, loggamma
import numpy as np
from math import log
from numpy.random import random_sample
import sys
from math import fabs
from scipy.stats import nbinom

class NegBin():
    """Negative Binomial distribution (NB1) with continuous parameter r,
    for NB1 see 
    Noriszura Ismail and Abdul Aziz Jemain. 
    Handling Overdispersion with Negative Binomial and Generalized Poisson Regression Models. 
    
    Implementation based on
    
    - http://stackoverflow.com/questions/11373192/generating-discrete-random-variables-with-specified-weights-using-scipy-or-numpy
    - http://www.nehalemlabs.net/prototype/blog/2013/11/11/negative-binomial-with-continuous-parameters-in-python/
    """
    
    def _get_value_log(self, x, mu, v):
        """log basic 2"""
        try:
            return loggamma(x+v) - loggamma(x+1) - loggamma(v) + v*log(v) - v*log(v+mu) + x*log(mu) - x*log(v+mu)
        except ValueError:
            #print('_get_value_log ValueError', x, mu, v, file=sys.stderr)
            return 1
    
    def _get_value(self, x, mu, v):
        try:
            return rf(v, x) / gamma(x+1) * ( v/float(v+mu) ) ** v * ( mu/float(v+mu) ) ** x
        except ValueError:
            print(x, mu, v, file=sys.stderr)
            return 1
            
    def __init__(self, mu, alpha):
        self.map_pdf = {}
        self.map_logpdf = {}
        self.bins = []
        mu = float(mu)
        
        self.alpha = alpha
        self.mu = mu
        
        self.nbin = np.frompyfunc(self._get_value, 3, 1)
        self.nbin_log = np.frompyfunc(self._get_value_log, 3, 1)
        
    def pdf(self, x):
        if not self.map_pdf.has_key(x):
            v_log = self.nbin(x, self.mu, 1./self.alpha)
            self.map_pdf[x] = v_log
        
        return self.map_pdf[x]
    
    def logpdf(self, x):
        if not self.map_logpdf.has_key(x):
            v_log = self.nbin_log(x, self.mu, 1./self.alpha)
            self.map_logpdf[x] = v_log
        
        return self.map_logpdf[x]
        
    def rvs(self):
        if not self.bins:
            probs = []
            i = 0
            v_old = -1
            while True:
                v = self.nbin(i, self.mu, 1./self.alpha)
                probs.append(v)
                
                i += 1
                if fabs(v_old - v) < 10**-10:
                    break
                v_old = v
            self.bins = map(lambda x: float(x), np.add.accumulate(probs))
        return np.digitize(random_sample(1), self.bins)[0]

if __name__ == '__main__':
    neg_bin = NegBin(0.1, 0.00000000001)
    s=0
    ew = 0
    distr = {'n': 10, 'p': 0.1}
    
    for i in range(10):
        #v = neg_bin.pdf(i)
        v = nbinom.logpmf(i, distr['n'], distr['p'])
        #v_log = neg_bin.logpdf(i)
        #s += v
        #ew += i * v
        #print(i, v, log(v), v_log, sep='\t')
        print(i, v, sep='\t')
    #print(s, ew)



"""
helper codes
===================
some other extra codes
"""

from GenomicRegion import GenomicRegion
from GenomicRegionSet import GenomicRegionSet

def get_chrom_sizes_as_genomicregionset(chrom_size_path):
    regionset = GenomicRegionSet('')
    with open(chrom_size_path) as f:
        for line in f:
            line = line.strip()
            line = line.split('\t')
            chrom, end = line[0], int(line[1])
            regionset.add(GenomicRegion(chrom=chrom, initial=0, final=end))
    
    return regionset

def pretty(d, indent=0):
    for key, value in d.iteritems():
        print('\t' * indent + str(key))
        if isinstance(value, dict):
            pretty(value, indent+1)
        else:
            for t in value:
                print('\t' * (indent+1) + str(t))



###################################################################################################
# Libraries
###################################################################################################

# Python
from __future__ import print_function

# Internal
from rgt.GenomicRegionSet import GenomicRegionSet
from rgt.GenomicRegion import GenomicRegion

# External
try:
    import MOODS.tools
    import MOODS.scan
except:
    import MOODS


###################################################################################################
# Functions
###################################################################################################

def match_single(motif, sequence, genomic_region, unique_threshold=None, normalize_bitscore=True, sort=False):
    """
    Performs motif matching given sequence and the motif.pssm passed as parameter.
    The genomic_region is needed to evaluate the correct binding position.
    Please note that the arguments should be passed as a list, to allow for parallelization
    mapping function.

    Keyword arguments:
    motif -- TODO.
    sequence -- A DNA sequence (string).
    genomic_region -- A GenomicRegion.
    output_file -- TODO.  
    unique_threshold -- If this argument is provided, the motif search will be made using a threshold of 0 and
                        then accepting only the motif matches with bitscore/motif_length >= unique_threshold.
        
    Return:
    Print MPBSs to output_file.
    """

    # Establishing threshold
    if unique_threshold:
        current_threshold = 0.0
        eval_threshold = unique_threshold
        motif_max = motif.max / motif.len
    else:
        current_threshold = motif.threshold
        eval_threshold = motif.threshold
        motif_max = motif.max

    # Performing motif matching
    try:
        # old MOODS version
        results = MOODS.search(sequence, [motif.pssm_list], current_threshold,
                               absolute_threshold=True, both_strands=True)
    except:
        # TODO: we can expand this to use bg from sequence, for example,
        # or from organism.
        bg = MOODS.tools.flat_bg(4)
        results = MOODS.scan.scan_dna(sequence, [motif.pssm_list], bg, [current_threshold], 7)

    grs = GenomicRegionSet("mpbs")

    for search_result in results:
        for r in search_result:
            try:
                position = r.pos
                score = r.score
            except:
                (position, score) = r

            # Verifying unique threshold acceptance
            if unique_threshold and score/motif.len < unique_threshold:
                continue

            # If match forward strand
            if position >= 0:
                p1 = genomic_region.initial + position
                strand = "+"
            # If match reverse strand
            elif not motif.is_palindrome:
                p1 = genomic_region.initial - position
                strand = "-"
            else:
                continue

            # Evaluating p2
            p2 = p1 + motif.len

            # Evaluating score (integer between 0 and 1000 -- needed for bigbed transformation)
            if normalize_bitscore:
                # Normalized bitscore = standardize to integer between 0 and 1000 (needed for bigbed transformation)
                if motif_max > eval_threshold:
                    norm_score = int(((score - eval_threshold) * 1000.0) / (motif_max - eval_threshold))
                else:
                    norm_score = 1000
            else:
                # Keep the original bitscore
                if unique_threshold:
                    norm_score = score/motif.len
                else:
                    norm_score = score

            grs.add(GenomicRegion(genomic_region.chrom, int(p1), int(p2),
                                  name=motif.name, orientation=strand, data=str(norm_score)))

    if sort:
        grs.sort()

    return grs



###################################################################################################
# Libraries
###################################################################################################

# Python
import os
import numpy as np
from pysam import Fastafile
from Bio import motifs
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import pyx

# Internal
from ..Util import GenomeData
from signalProcessing import GenomicSignal
from rgt.GenomicRegionSet import GenomicRegionSet
from biasTable import BiasTable
from ..Util import AuxiliaryFunctions



class Plot:
    """

    """
    def __init__(self, bam_file, motif_file, motif_name, window_size,
                 atac_downstream_ext, atac_upstream_ext, atac_forward_shift, atac_reverse_shift,
                 initial_clip, organism, bias_table, k_nb, protection_score,
                 strands_specific, output_loc):
        self.bam_file = bam_file
        self.motif_file = motif_file
        self.motif_name = motif_name
        self.window_size = window_size
        self.atac_downstream_ext = atac_downstream_ext
        self.atac_upstream_ext = atac_upstream_ext
        self.atac_forward_shift = atac_forward_shift
        self.atac_reverse_shift = atac_reverse_shift
        self.initial_clip = initial_clip
        self.organism = organism
        self.bias_table = bias_table
        self.k_nb = k_nb
        self.strands_specific = strands_specific
        self.protection_score = protection_score
        self.output_loc = output_loc

    def line(self):
        signal = GenomicSignal(self.bam_file)
        signal.load_sg_coefs(slope_window_size=9)
        bias_table = BiasTable()
        bias_table_list = self.bias_table.split(",")
        table = bias_table.load_table(table_file_name_F=bias_table_list[0],
                                      table_file_name_R=bias_table_list[1])
        genome_data = GenomeData(self.organism)
        fasta = Fastafile(genome_data.get_genome())
        pwm_dict = dict([("A", [0.0] * self.window_size), ("C", [0.0] * self.window_size),
                        ("G", [0.0] * self.window_size), ("T", [0.0] * self.window_size),
                        ("N", [0.0] * self.window_size)])


        mean_raw_signal = np.zeros(self.window_size)
        mean_bc_signal = np.zeros(self.window_size)
        mean_raw_signal_f = np.zeros(self.window_size)
        mean_bc_signal_f = np.zeros(self.window_size)
        mean_raw_signal_r = np.zeros(self.window_size)
        mean_bc_signal_r = np.zeros(self.window_size)

        mean_bias_signal_f = np.zeros(self.window_size)
        mean_bias_signal_r = np.zeros(self.window_size)
        num_sites = 0

        mpbs_regions = GenomicRegionSet("Motif Predicted Binding Sites")
        mpbs_regions.read_bed(self.motif_file)

        total_nc_signal = 0
        total_nl_signal = 0
        total_nr_signal = 0

        for region in mpbs_regions:
            if str(region.name).split(":")[-1] == "Y":
                num_sites += 1
                # Extend by 50 bp
                mid = (region.initial + region.final) / 2
                p1 = mid - (self.window_size / 2)
                p2 = mid + (self.window_size / 2)

                if not self.strands_specific:
                    # Fetch raw signal
                    raw_signal, _ = signal.get_signal(ref=region.chrom, start=p1, end=p2,
                                                  downstream_ext=self.atac_downstream_ext,
                                                  upstream_ext=self.atac_upstream_ext,
                                                  forward_shift=self.atac_forward_shift,
                                                  reverse_shift=self.atac_reverse_shift,
                                                  genome_file_name=genome_data.get_genome())

                    mean_raw_signal = np.add(mean_raw_signal, raw_signal)

                    # Fetch bias correction signal
                    bc_signal, _ = signal.get_signal(ref=region.chrom, start=p1, end=p2,
                                                     bias_table=table,
                                                     downstream_ext=self.atac_downstream_ext,
                                                     upstream_ext=self.atac_upstream_ext,
                                                     forward_shift=self.atac_forward_shift,
                                                     reverse_shift=self.atac_reverse_shift,
                                                     genome_file_name=genome_data.get_genome())

                    mean_bc_signal = np.add(mean_bc_signal, bc_signal)
                else:
                    raw_signal_f, _, raw_signal_r, _ =  signal.get_signal_per_strand(ref=region.chrom, start=p1, end=p2,
                                                                        downstream_ext=self.atac_downstream_ext,
                                                                        upstream_ext=self.atac_upstream_ext,
                                                                        forward_shift=self.atac_forward_shift,
                                                                        reverse_shift=self.atac_reverse_shift,
                                                                        genome_file_name=genome_data.get_genome())
                    mean_raw_signal_f = np.add(mean_raw_signal_f, raw_signal_f)
                    mean_raw_signal_r = np.add(mean_raw_signal_r, raw_signal_r)

                    bc_signal_f, _, bc_signal_r, _ = signal.get_signal_per_strand(ref=region.chrom, start=p1, end=p2,
                                                                                  bias_table=table,
                                                                                  downstream_ext=self.atac_downstream_ext,
                                                                                  upstream_ext=self.atac_upstream_ext,
                                                                                  forward_shift=self.atac_forward_shift,
                                                                                  reverse_shift=self.atac_reverse_shift,
                                                                                  genome_file_name=genome_data.get_genome())
                    mean_bc_signal_f = np.add(mean_bc_signal_f, bc_signal_f)
                    mean_bc_signal_r = np.add(mean_bc_signal_r, bc_signal_r)

                # Update pwm
                aux_plus = 1
                dna_seq = str(fasta.fetch(region.chrom, p1, p2)).upper()
                if (region.final - region.initial) % 2 == 0:
                    aux_plus = 0
                dna_seq_rev = AuxiliaryFunctions.revcomp(str(fasta.fetch(region.chrom,
                                                                         p1 + aux_plus, p2 + aux_plus)).upper())
                if region.orientation == "+":
                    for i in range(0, len(dna_seq)):
                        pwm_dict[dna_seq[i]][i] += 1
                elif region.orientation == "-":
                    for i in range(0, len(dna_seq_rev)):
                        pwm_dict[dna_seq_rev[i]][i] += 1

                # Create bias signal
                bias_table_f = table[0]
                bias_table_r = table[1]
                self.k_nb = len(bias_table_f.keys()[0])
                bias_signal_f = []
                bias_signal_r = []
                p1_wk = p1 - int(self.k_nb / 2)
                p2_wk = p2 + int(self.k_nb / 2)
                dna_seq = str(fasta.fetch(region.chrom, p1_wk, p2_wk - 1)).upper()
                dna_seq_rev = AuxiliaryFunctions.revcomp(str(fasta.fetch(region.chrom, p1_wk, p2_wk + 1)).upper())
                for i in range(int(self.k_nb / 2), len(dna_seq) - int(self.k_nb / 2) + 1):
                    fseq = dna_seq[i - int(self.k_nb / 2):i + int(self.k_nb / 2)]
                    rseq = dna_seq_rev[len(dna_seq) - int(self.k_nb / 2) - i:len(dna_seq) + int(self.k_nb / 2) - i]
                    try:
                        bias_signal_f.append(bias_table_f[fseq])
                    except Exception:
                        bias_signal_f.append(1)
                    try:
                        bias_signal_r.append(bias_table_r[rseq])
                    except Exception:
                        bias_signal_r.append(1)

                mean_bias_signal_f = np.add(mean_bias_signal_f, np.array(bias_signal_f))
                mean_bias_signal_r = np.add(mean_bias_signal_r, np.array(bias_signal_r))

                if self.protection_score:
                    # signal in the center of the MPBS
                    p1 = region.initial
                    p2 = region.final
                    nc_signal, _ = signal.get_signal(ref=region.chrom, start=p1, end=p2,
                                                     bias_table=table,
                                                     downstream_ext=self.atac_downstream_ext,
                                                     upstream_ext=self.atac_upstream_ext,
                                                     forward_shift=self.atac_forward_shift,
                                                     reverse_shift=self.atac_reverse_shift,
                                                     genome_file_name=genome_data.get_genome())
                    total_nc_signal += sum(nc_signal)
                    p1 = region.final
                    p2 = 2 * region.final - region.initial
                    nr_signal, _ = signal.get_signal(ref=region.chrom, start=p1, end=p2,
                                                     bias_table=table,
                                                     downstream_ext=self.atac_downstream_ext,
                                                     upstream_ext=self.atac_upstream_ext,
                                                     forward_shift=self.atac_forward_shift,
                                                     reverse_shift=self.atac_reverse_shift,
                                                     genome_file_name=genome_data.get_genome())
                    total_nr_signal += sum(nr_signal)
                    p1 = 2 * region.initial - region.final
                    p2 = region.final
                    nl_signal, _ = signal.get_signal(ref=region.chrom, start=p1, end=p2,
                                                     bias_table=table,
                                                     downstream_ext=self.atac_downstream_ext,
                                                     upstream_ext=self.atac_upstream_ext,
                                                     forward_shift=self.atac_forward_shift,
                                                     reverse_shift=self.atac_reverse_shift,
                                                     genome_file_name=genome_data.get_genome())
                    total_nl_signal += sum(nl_signal)


        mean_raw_signal = mean_raw_signal / num_sites
        mean_bc_signal = mean_bc_signal / num_sites

        mean_raw_signal_f = mean_raw_signal_f / num_sites
        mean_raw_signal_r = mean_raw_signal_r / num_sites
        mean_bc_signal_f = mean_bc_signal_f / num_sites
        mean_bc_signal_r = mean_bc_signal_r / num_sites

        mean_bias_signal_f = mean_bias_signal_f / num_sites
        mean_bias_signal_r = mean_bias_signal_r / num_sites

        protection_score = (total_nl_signal + total_nr_signal - 2 * total_nc_signal) / (2 * num_sites)

        # Output PWM and create logo
        pwm_fname = os.path.join(self.output_loc, "{}.pwm".format(self.motif_name))
        pwm_file = open(pwm_fname,"w")
        for e in ["A","C","G","T"]:
            pwm_file.write(" ".join([str(int(f)) for f in pwm_dict[e]])+"\n")
        pwm_file.close()

        logo_fname = os.path.join(self.output_loc, "{}.logo.eps".format(self.motif_name))
        pwm = motifs.read(open(pwm_fname), "pfm")
        pwm.weblogo(logo_fname, format="eps", stack_width="large", stacks_per_line="100",
                    color_scheme="color_classic", unit_name="", show_errorbars=False, logo_title="",
                    show_xaxis=False, xaxis_label="", show_yaxis=False, yaxis_label="",
                    show_fineprint=False, show_ends=False)

        # Output the raw, bias corrected signal and protection score
        output_fname = os.path.join(self.output_loc, "{}.txt".format(self.motif_name))
        output_file = open(output_fname, "w")
        if not self.strands_specific:
            output_file.write("raw signal: \n" + np.array_str(mean_raw_signal) + "\n")
            output_file.write("bias corrected signal: \n" + np.array_str(mean_bc_signal) + "\n")
        else:
            output_file.write("raw forward signal: \n" + np.array_str(mean_raw_signal_f) + "\n")
            output_file.write("bias corrected forward signal: \n" + np.array_str(mean_bc_signal_f) + "\n")
            output_file.write("raw reverse signal: \n" + np.array_str(mean_raw_signal_r) + "\n")
            output_file.write("bias reverse corrected signal: \n" + np.array_str(mean_bc_signal_r) + "\n")
        output_file.write("forward bias signal: \n" + np.array_str(mean_bias_signal_f) + "\n")
        output_file.write("reverse bias signal: \n" + np.array_str(mean_bias_signal_r) + "\n")
        if self.protection_score:
            output_file.write("protection score: \n" + str(protection_score) + "\n")
        output_file.close()

        if self.strands_specific:
            fig, (ax1, ax2, ax3) = plt.subplots(3, figsize=(12.0, 10.0))
        else:
            fig, (ax1, ax2) = plt.subplots(2)
        x = np.linspace(-50, 49, num=self.window_size)

        ax1.plot(x, mean_bias_signal_f, color='red', label='Forward')
        ax1.plot(x, mean_bias_signal_r, color='blue', label='Reverse')

        ax1.xaxis.set_ticks_position('bottom')
        ax1.yaxis.set_ticks_position('left')
        ax1.spines['top'].set_visible(False)
        ax1.spines['right'].set_visible(False)
        ax1.spines['left'].set_position(('outward', 15))
        ax1.spines['bottom'].set_position(('outward', 5))
        ax1.tick_params(direction='out')

        ax1.set_xticks([-50, -40, -30, -20, -10, 0, 10, 20, 30, 40, 49])
        ax1.set_xticklabels(['-50', '-40', '-30', '-20', '-10', '0', '10', '20', '30', '40', '49'])
        min_bias_signal = min(min(mean_bias_signal_f), min(mean_bias_signal_r))
        max_bias_signal = max(max(mean_bias_signal_f), max(mean_bias_signal_r))
        ax1.set_yticks([min_bias_signal, max_bias_signal])
        ax1.set_yticklabels([str(round(min_bias_signal,2)), str(round(max_bias_signal,2))], rotation=90)

        ax1.text(-48, max_bias_signal, '# Sites = {}'.format(str(num_sites)), fontweight='bold')
        ax1.set_title(self.motif_name, fontweight='bold')
        ax1.set_xlim(-50, 49)
        ax1.set_ylim([min_bias_signal, max_bias_signal])
        ax1.legend(loc="upper right", frameon=False)
        ax1.set_ylabel("Average Bias \nSignal", rotation=90, fontweight='bold')

        if not self.strands_specific:
            mean_raw_signal = self.standardize(mean_raw_signal)
            mean_bc_signal = self.standardize(mean_bc_signal)
            ax2.plot(x, mean_raw_signal, color='red', label='Uncorrected')
            ax2.plot(x, mean_bc_signal, color='green', label='Corrected')
        else:
            mean_raw_signal_f = self.standardize(mean_raw_signal_f)
            mean_raw_signal_r = self.standardize(mean_raw_signal_r)
            mean_bc_signal_f = self.standardize(mean_bc_signal_f)
            mean_bc_signal_r = self.standardize(mean_bc_signal_r)
            ax2.plot(x, mean_raw_signal_f, color='red', label='Forward')
            ax2.plot(x, mean_raw_signal_r, color='green', label='Reverse')
            ax3.plot(x, mean_bc_signal_f, color='red', label='Forward')
            ax3.plot(x, mean_bc_signal_r, color='green', label='Reverse')

        ax2.xaxis.set_ticks_position('bottom')
        ax2.yaxis.set_ticks_position('left')
        ax2.spines['top'].set_visible(False)
        ax2.spines['right'].set_visible(False)
        ax2.spines['left'].set_position(('outward', 15))
        ax2.tick_params(direction='out')
        ax2.set_xticks([-50, -40, -30, -20, -10, 0, 10, 20, 30, 40, 49])
        ax2.set_xticklabels(['-50', '-40', '-30', '-20', '-10', '0', '10', '20', '30', '40', '49'])
        ax2.set_yticks([0, 1])
        ax2.set_yticklabels([str(0), str(1)], rotation=90)
        ax2.set_xlim(-50, 49)
        ax2.set_ylim([0, 1])

        if not self.strands_specific:
            ax2.spines['bottom'].set_position(('outward', 40))
            ax2.set_xlabel("Coordinates from Motif Center", fontweight='bold')
            ax2.set_ylabel("Average ATAC-seq \nSignal", rotation=90, fontweight='bold')
            ax2.legend(loc="center", frameon=False, bbox_to_anchor=(0.85, 0.06))
        else:
            ax2.spines['bottom'].set_position(('outward', 5))
            ax2.set_ylabel("Average ATAC-seq \n Uncorrected Signal", rotation=90, fontweight='bold')
            ax2.legend(loc="lower right", frameon=False)

            ax3.xaxis.set_ticks_position('bottom')
            ax3.yaxis.set_ticks_position('left')
            ax3.spines['top'].set_visible(False)
            ax3.spines['right'].set_visible(False)
            ax3.spines['left'].set_position(('outward', 15))
            ax3.tick_params(direction='out')
            ax3.set_xticks([-50, -40, -30, -20, -10, 0, 10, 20, 30, 40, 49])
            ax3.set_xticklabels(['-50', '-40', '-30', '-20', '-10', '0', '10', '20', '30', '40', '49'])
            ax3.set_yticks([0, 1])
            ax3.set_yticklabels([str(0), str(1)], rotation=90)
            ax3.set_xlim(-50, 49)
            ax3.set_ylim([0, 1])
            ax3.legend(loc="lower right", frameon=False)
            ax3.spines['bottom'].set_position(('outward', 40))
            ax3.set_xlabel("Coordinates from Motif Center", fontweight='bold')
            ax3.set_ylabel("Average ATAC-seq \n Corrected Signal", rotation=90, fontweight='bold')
            ax3.text(-48, 0.05, '# K-mer = {}\n# Forward Shift = {}'.format(str(self.k_nb), str(self.atac_forward_shift)),
                     fontweight='bold')

        figure_name = os.path.join(self.output_loc, "{}.line.eps".format(self.motif_name))
        fig.subplots_adjust(bottom=.2, hspace=.5)
        fig.tight_layout()
        fig.savefig(figure_name, format="eps", dpi=300)

        # Creating canvas and printing eps / pdf with merged results
        output_fname = os.path.join(self.output_loc, "{}.eps".format(self.motif_name))
        c = pyx.canvas.canvas()
        c.insert(pyx.epsfile.epsfile(0, 0, figure_name, scale=1.0))
        if self.strands_specific:
            c.insert(pyx.epsfile.epsfile(2.76, 1.58, logo_fname, width=27.2, height=2.45))
        else:
            c.insert(pyx.epsfile.epsfile(2.5, 1.54, logo_fname, width=16, height=1.75))
        c.writeEPSfile(output_fname)
        os.system("epstopdf " + figure_name)
        os.system("epstopdf " + logo_fname)
        os.system("epstopdf " + output_fname)

    def standardize(self, vector):
        maxN = max(vector)
        minN = min(vector)
        return [(e - minN) / (maxN - minN) for e in vector]


"""
CoverageSet
===================
CoverageSet represents the coverage data of a GenomicRegionSet.

"""

from __future__ import print_function
import os
import sys
import pysam
import numpy as np


class CoverageSet:
    """*Keyword arguments:*

        - name -- names.
        - genomicRegions -- instance of GenomicRegionSet
    """
    
    def __init__(self, name, GenomicRegionSet):
        """Initialize CoverageSet <name>."""
        self.name = name
        self.genomicRegions = GenomicRegionSet
        self.coverage = [] #coverage data for genomicRegions
        self.binsize = 100
        self.mapped_reads = None #number of mapped read
        self.reads = None #number of reads
        self.stepsize = 50
    
    def subtract(self, cs):
        """Substract CoverageSet <cs>.
        
        *Keyword arguments:*
        
        - cs -- instance of CoverageSet
        
        .. note::
            negative values are set to 0.
        """
        
        cs_chroms = cs.genomicRegions.get_chrom()
        assert len(cs_chroms) == len(set(cs_chroms)) #no double entries
        assert len(self.genomicRegions.get_chrom()) == len(set(self.genomicRegions.get_chrom()))
        
        i = 0
        for c in self.genomicRegions.get_chrom(): #c corresponds to self.coverage[i]
            try:
                j = cs_chroms.index(c)
                assert len(self.coverage[i]) == len(cs.coverage[j])
                self.coverage[i] -= cs.coverage[j]
                self.coverage[i] = self.coverage[i].clip(0, max(max(self.coverage[i]), 0)) #neg. values to 0
            except ValueError:
                pass
            i += 1
    
    def add(self, cs):
        """Add CoverageSet <cs>.
        
        *Keyword arguments:*
        
        - cs -- instance of CoverageSet, which is used to add up
        
        """
        cs_chroms = cs.genomicRegions.get_chrom()
        assert len(cs_chroms) == len(set(cs_chroms)) #no double entries
        assert len(self.genomicRegions.get_chrom()) == len(set(self.genomicRegions.get_chrom()))
        
        i = 0
        for c in self.genomicRegions.get_chrom(): #c corresponds to self.coverage[i]
            try:
                j = cs_chroms.index(c)
                assert len(self.coverage[i]) == len(cs.coverage[j])
                self.coverage[i] += cs.coverage[j]
            except ValueError:
                pass
            i += 1
    
    def scale(self, factor):
        """Scale coverage with <factor>.
        
        *Keyword arguments:*
        
        - factor -- float
        
        """
        for i in range(len(self.coverage)):
            self.coverage[i] = np.rint(self.coverage[i] * float(factor)).astype(int)

    def normRPM(self):
        """Normalize to read per million (RPM)."""
        if self.reads == 0:
            print("Error! The reads number is zero in "+self.name)
            print("** Please try to reindex the file by \'samtools index\'.")
            sys.exit(1)

        factor=1000000/float(self.reads)
        self.coverage = np.array(self.coverage)*factor
        try:
            self.transpose_cov1 = np.array(self.transpose_cov1) * factor
            self.transpose_cov2 = np.array(self.transpose_cov2) * factor
        except:
            pass


    def write_bed(self, filename, zero=False):
        """Output coverage in BED format. 
        
        *Keyword arguments:*
        
        - filename -- filepath
        - zero -- boolean
        
        .. note:: If zero=True, coverage of zero is output as well. This may cause large output files!
        
        """
        
        with open(filename, 'w') as f:
            i = 0
            for region in self.genomicRegions:
                c = self.coverage[i]
                i += 1
                for j in range(len(c)):
                    if zero:
                        print(region.chrom, j * self.stepsize + ((self.binsize-self.stepsize)/2) + region.initial, \
                              j * self.stepsize + ((self.binsize+self.stepsize)/2) + region.initial, c[j], sep='\t', file=f)
                    else:
                        if c[j] != 0:
                            print(region.chrom, j * self.stepsize + ((self.binsize-self.stepsize)/2) + region.initial, \
                                  j * self.stepsize + ((self.binsize+self.stepsize)/2) + region.initial, c[j], sep='\t', file=f)
                        
    def write_wig(self, filename):
        """Output coverage in wig format. 
        
        *Keyword arguments:*
        
        - filename -- filepath        
        """
        f = open(filename, 'w')
        i = 0
        for region in self.genomicRegions:
            print('variableStep chrom=' + str(region.chrom) + ' span=' +str(self.stepsize), file=f)
            c = self.coverage[i]
            i += 1
            for j in range(len(c)):
                if c[j] != 0:
                    print(j * self.stepsize + ((self.binsize-self.stepsize)/2), c[j], file=f)
        f.close()
    
    def write_bigwig(self, filename, chrom_file, end=True, save_wig=False):
        """Output coverage in bigwig format. 
        
        The path to the chromosome size file <chrom_file> is required. This file is tab-separated and assigns
        a chromosome to its size.
        
        *Keyword arguments:*
        
        - filename -- filepath
        - chrom_file -- chromosome size file
        - end -- boolean
        - save_wig -- boolean, if set, wig file is also saved.
        
        .. warning:: Parameter <end> is deprecated! Please do not use it.
        
        .. note:: The <save_wig> option may cause large output files 
        
        """

        tmp_path = filename + '.wig'
        self.write_wig(tmp_path)
        t = ['wigToBigWig', "-clip", tmp_path, chrom_file, filename]
        c = " ".join(t)
        os.system(c)

        if not save_wig:
            os.remove(tmp_path)
    
    def _init_read_number(self, bamFile):
        """Compute number of reads and number of mapped reads for CoverageSet"""
        # XXX ToDo add number of mapped reads in all cases
        # try:
        from distutils.version import LooseVersion
        if LooseVersion("0.9.0") <= LooseVersion(pysam.__version__):
            a = pysam.idxstats(bamFile)
            mapped_reads = sum([int(el.split('\t')[2]) for el in a.split('\n')[:len(a.split('\n'))-1]])
            unmapped_read = sum([int(el.split('\t')[3]) for el in a.split('\n')[:len(a.split('\n'))-1]])
            self.reads = mapped_reads + unmapped_read
            self.mapped_reads = mapped_reads
        else:
            self.reads = reduce(lambda x, y: x + y, [ eval('+'.join(l.rstrip('\n').split('\t')[2:]) ) for l in pysam.idxstats(bamFile)])
            self.mapped_reads = None
        # except:
        #     self.reads = None
        #     self.mapped_reads = None
    
    def coverage_from_genomicset(self, bamFile, readSize=200, strand_specific=False):

        """Compute coverage based on the class variable <genomicRegions>. 
        
        Iterate over each GenomicRegion in class variable genomicRegions (GenomicRegionSet) and set coverage to the number of reads falling into the GenomicRegion.
              
        *Keyword arguments:*
        
        - bamFile -- path to bam file
        - readSize -- used read size
        - strand_specific -- calculate the coverage from the reads with the same orientation with the region
        
        *Output:*
        
        Class variable <coverage>: a list where the elements correspond to the GenomicRegion. The list elements give
        the number of reads falling into the GenomicRegion.
        
        """
        
        bam = pysam.Samfile(bamFile, "rb" )
        self._init_read_number(bamFile)
        
        cov=[0]*len(self.genomicRegions)
        for i,region in enumerate(self.genomicRegions):
            
            try:
                bin_start = max(0, region.initial-readSize)
                bin_end = region.final+readSize

                if not strand_specific:
                    for r in bam.fetch(region.chrom,bin_start,bin_end):
                        cov[i] += 1
                else:
                    for r in bam.fetch(region.chrom,bin_start,bin_end):
                        # print(region.orientation)
                        # print(r.is_reverse)
                        if region.orientation == "+" and not r.is_reverse: cov[i] += 1
                        elif region.orientation == "-" and r.is_reverse: cov[i] += 1
                
            except:
                print("\tSkip: "+region.toString())

        self.coverage = cov 
        self.coverageOrig = cov

    def _get_bedinfo(self, l):
        if len(l) > 1:
            l.strip()
            l = l.split('\t')
            return l[0], int(l[1]), int(l[2]), True
        else:
            return -1, -1, -1, False

    
    def coverage_from_bam(self, bam_file, extension_size=200, binsize=100, stepsize=50, rmdup=False,
                          maxdup=None, mask_file=None, paired_reads=False,
                          get_strand_info=False, get_sense_info=False, no_gaps=False):
        """Compute coverage based on GenomicRegionSet. 
        
        Iterate over each GenomicRegion in class variable genomicRegions (GenomicRegionSet). The GenomicRegion is divided into consecutive bins with lenth <binsize>.
        A sliding-window approach with a stepsize of <stepsize> generates the coverage signal.
              
        *Keyword arguments:*
        
        - bam_file -- path to bam file
        - extension_size -- used read size
        - binsize -- size of bins
        - stepsize -- stepsize for the window-based approach to generat the signal
        - rmdup -- remove dupliacted reads (reads with same starting coordinate)
        - maxdup -- define the maximum count for the dupliacted reads (0: remove all;-1:no limit)
        - mask_file -- ignore region described in <mask_file> (tab-separated: chrom, start, end)
        - get_strand_info -- compute strand information for each bin
        
        
        *Output:*
        
        - Class variable <coverage>: a list of lists: the elements correspond a GenomicRegion. This list gives the coverage of each bin.
        - Class variable <overall_cov>: a list: concatenation of class variable <coverage>.
        - If option <get_strand_info> is set, a numpy array class variable  <cov_strand_all> of tuples. The tuples give the number of forward and backward reads for each bin.
        
        *Example:*
        
        First, we compute a GenomicRegionSet that covers the entire mouse genome mm9. We use the annotation of RGT to compute the variable <regionset>::
            
            >>>from rgt.Util import GenomeData
            >>>from rgt.helper import get_chrom_sizes_as_genomicregionset
            
            >>>g = GenomeData('mm9')
            >>>regionset = get_chrom_sizes_as_genomicregionset(g.get_chromosome_sizes())
        
        Next, we load the CoverageSet class from RGT and initialize it with the variable <regionset>. Finally, we compute the coverage based on <bamfile>::
        
            >>>from rgt.CoverageSet import CoverageSet
            >>>cov = CoverageSet('IP coverage', regionset)
            >>>cov.coverage_from_bam(bam_file=bamfile, extension_size=200)
        
        We can now access <cov>::
        
            >>>from __future__ import print_function
            >>>from numpy import sum
            >>>print(cov.overall_cov[cov.overall_cov>0][:10])
            [1 1 1 1 1 2 2 2 2 1]
            
            >>>print(len(cov.overall_cov))
            54515813
        
        .. note::
        
         the length of the <overall_cov> equals 54515813, as we take the entire genome into account, but use a the default stepsize of 50 for segmentation. 
        
        """

        if len(self.genomicRegions) == 0:
            return
        log_aver = False
        self.binsize = binsize
        self.stepsize = stepsize
        self.coverage = []
        
        bam = pysam.Samfile(bam_file, "rb" )
        
        for read in bam.fetch():
            fragment_size = read.rlen + extension_size
            break

        self._init_read_number(bam_file)
        
        #check whether one should mask
        next_it = True
        if mask_file is not None and os.path.exists(mask_file):
            mask = True
            f = open(mask_file, 'r')
            c_help, s_help, e_help = self.genomicRegions.sequences[0].chrom, -1, -1
        else:
            mask = False
        
        chrom_regions = [r.chrom for r in self.genomicRegions.sequences] #chroms by regions
        
        if get_strand_info:
            self.cov_strand_all = []
        if get_sense_info:
            self.cov_sense_all = []
        
        for region in self.genomicRegions:
            cov = [0] * (len(region) / stepsize)
            
            if get_strand_info:
                cov_strand = [[0,0]] * (len(region) / stepsize)
                strand_info = {}
            if get_sense_info:
                cov_sense = [[0,0]] * (len(region) / stepsize)
                sense_info = {}
            
            positions = []
            j = 0
            read_length = -1
            try:
                for read in bam.fetch(region.chrom, max(0, region.initial-fragment_size), region.final+fragment_size):
                    if len(read.get_blocks()) > 1 and no_gaps: continue # ignore sliced reads
                    j += 1
                    read_length = read.rlen
                    if not read.is_unmapped:
                        # pos = read.pos - extension_size if read.is_reverse else read.pos
                        # pos_help = read.pos - read.qlen if read.is_reverse else read.pos
                        pos = read.pos - extension_size if read.is_reverse else read.pos
                        pos_help = read.pos - read.qlen if read.is_reverse else read.pos

                        within_gap = False
                        if no_gaps:
                            blocks = read.get_blocks()
                            if len(blocks) > 1:
                                # print(read.is_reverse)
                                # print(read.pos)
                                # print(blocks)
                                for b_ind in range(len(blocks) - 1):
                                    # print([blocks[b_ind][1], read.pos, blocks[b_ind+1][0]])
                                    if blocks[b_ind][1] <= read.pos < blocks[b_ind+1][0]:
                                        within_gap = True

                        #if position in mask region, then ignore
                        if mask:
                            while next_it and c_help not in chrom_regions: #do not consider this deadzone
                                c_help, s_help, e_help, next_it = self._get_bedinfo(f.readline())
                            # deadzones behind, go further
                            if c_help != -1 and chrom_regions.index(region.chrom) >= chrom_regions.index(c_help):
                                while next_it and c_help != region.chrom: #get right chromosome
                                    c_help, s_help, e_help, next_it = self._get_bedinfo(f.readline())
                            while next_it and e_help <= pos_help and c_help == region.chrom: #check right position
                                c_help, s_help, e_help, next_it = self._get_bedinfo(f.readline())
                            if next_it and s_help <= pos_help and c_help == region.chrom:
                                continue #pos in mask region
                        if within_gap: continue
                        else: positions.append(pos)

                        if get_strand_info:
                            if pos not in strand_info:
                                strand_info[pos] = (1,0) if not read.is_reverse else (0,1)
                        if get_sense_info:
                            if pos not in sense_info:
                                if paired_reads and not read.is_read1:
                                    continue
                                else:
                                    if region.orientation == "+":
                                        sense_info[pos] = (1,0) if read.is_reverse else (0,1)
                                    elif region.orientation == "-":
                                        sense_info[pos] = (1,0) if not read.is_reverse else (0,1)
            except ValueError as e:
                print("warning: {}".format(e))
                pass

            # if maxdup == -1: # No limit
            # elif maxdup == 0: # Remove all duplicates
            # else: # 

            if rmdup:
                positions = list(set(positions))
                
            positions.sort()
            positions.reverse()
            i = 0
            while positions:
                win_s = max(0, i * stepsize - binsize*0.5) + region.initial
                win_e = i * stepsize + binsize*0.5 + region.initial 
                c = 0
                if get_strand_info:
                    sum_strand_info = [0,0]
                if get_sense_info:
                    sum_sense_info = [0,0]
                
                taken = []
                while True:
                    s = positions.pop()
                    
                    taken.append(s)
                    if s < win_e: #read within window
                        c += 1
                        if get_strand_info:
                            sum_strand_info[0] += strand_info[s][0]
                            sum_strand_info[1] += strand_info[s][1]
                        if get_sense_info:
                            try:
                                sum_sense_info[0] += sense_info[s][0]
                                sum_sense_info[1] += sense_info[s][1]
                            except:
                                pass
                    if s >= win_e or not positions:
                        taken.reverse()
                        for s in taken:
                            if s + extension_size + read_length >= win_s: #consider read in next iteration
                                positions.append(s)
                            else:
                                break #as taken decreases monotonously
                        taken = []
                        break
                if i < len(cov):
                    cov[i] = c
                    if get_strand_info:
                        cov_strand[i] = sum_strand_info
                    if get_sense_info:
                        cov_sense[i] = sum_sense_info
                i += 1

            if not log_aver:
                self.coverage.append(np.array(cov))
            else:
                cov = [x + 1 for x in cov]
                self.coverage.append(np.log(np.array(cov)))

            if get_strand_info:
                self.cov_strand_all.append(np.array(cov_strand))
            if get_sense_info:
                self.cov_sense_all.append(np.array(cov_sense))
            # print(np.array(cov_sense))
            
        self.coverageorig = self.coverage[:]
        self.overall_cov = reduce(lambda x,y: np.concatenate((x,y)), [self.coverage[i] for i in range(len(self.genomicRegions))])
        if mask: f.close()


    def array_transpose(self, flip=False):
        """Transpose the arrays in strand coverage"""
        self.transpose_cov1 = []
        self.transpose_cov2 = []
        # print(self.coverage)
        for a in self.cov_sense_all:
            if flip:
                # print(a[:, 0])
                # print(a[:, 1])
                a1 = np.transpose(a[:, 0])
                a1.shape = (a1.shape[0],1)
                self.transpose_cov1.append(np.fliplr(a1))
                a2 = np.transpose(a[:, 0])
                a2.shape = (a2.shape[0], 1)
                self.transpose_cov2.append(np.fliplr(a2))
            else:
                # print(a[:, 0])
                # print(a[:, 1])
                a1 = np.transpose(a[:, 0])
                a1.shape = (a1.shape[0], 1)
                self.transpose_cov1.append(a1)
                a2 = np.transpose(a[:, 1])
                a2.shape = (a2.shape[0], 1)
                self.transpose_cov2.append(a2)
        self.transpose_cov1 = np.array(self.transpose_cov1)
        self.transpose_cov2 = np.array(self.transpose_cov2)
        # print(self.transpose_cov1)
        # print(self.transpose_cov2)



    def index2coordinates(self, index, regions):
        """Convert index of class variable <overall_cov> to genomic coordinates.
        
        *Keyword arguments:*
        
        - index -- index of <overall_cov> that is to be converted
        - regions -- instance of GenomicRegionSet the conversion is based on
        
        .. note:: In most of the cases, the parameter <regions> equals the GenomicRegionSet used for the initialization of the CoverageSet.
                
        *Output:*
        
        Triple which gives the chromosome, the start- and the end-coordinate of the bin associated to <index>.
        
        *Example:*
        
        Here, we give out the genomic regions of bins that exhibit a value higher than 10::
        
            >>>from rgt.CoverageSet import CoverageSet
            >>>cov = CoverageSet('IP coverage', regionset)
            >>>cov.coverage_from_bam(bam_file=bamfile, extension_size=200)
            >>>for i, el in enumerate(cov.overall_cov):
            >>>    if el > 10:
            >>>        chrom, s, e = cov.index2coordinates(i, regionset)
            >>>        print(chrom, s, e)
        
        """
        r = regions
        iter = r.__iter__()
        r = iter.next()
        sum = r.final
        last = 0
        i = 0
        while sum <= index * self.stepsize:
            last += len(self.coverage[i])
            try:
                r = iter.next()
            except StopIteration:
                sum += r.final
                i += 1
                break
            sum += r.final
            i += 1
        
        return r.chrom, (index-last) * self.stepsize, \
            min((index-last) * self.stepsize + self.stepsize, r.final)
    
    def coverage_from_bigwig(self, bigwig_file, stepsize=100):

        """Return list of arrays describing the coverage of each genomicRegions from <bigwig_file>.
        
        *Keyword arguments:*
        
        - bigwig_file -- path to bigwig file
        - stepsize -- used stepsize
        
        *Output:*
        
        Class variable <coverage>: a list where the elements correspond to the GenomicRegion. The list elements give
        the number of reads falling into the GenomicRegion.
        
        """
        try:
            from ngslib import BigWigFile
            self.coverage = []
            bwf = BigWigFile(bigwig_file)

            for gr in self.genomicRegions:
                depth = bwf.pileup(gr.chrom, max(0, int(gr.initial - stepsize / 2)),
                                   max(1, int(gr.final + stepsize / 2)))
                ds = [depth[d] for d in range(0, gr.final - gr.initial, stepsize)]
                self.coverage.append(np.array(ds))
            bwf.close()

        except ImportError, e:
            import pyBigWig
            self.coverage = []
            bwf = pyBigWig.open(bigwig_file)

            for gr in self.genomicRegions:
                steps = int(len(gr) / stepsize)
                ds = bwf.stats(gr.chrom, gr.initial, gr.final, type="mean", nBins=steps)
                ds = [ x if x else 0 for x in ds ]
                self.coverage.append( np.array(ds) )
            bwf.close()

    def phastCons46way_score(self, stepsize=100):
        """Load the phastCons46way bigwig files to fetch the scores as coverage.
        
        *Keyword arguments:*
        
        - stepsize -- used stepsize
        """
        self.coverage = []
        phastCons46way_dir = "/data/phastCons46way/"
        grs = self.genomicRegions.split_by_chromosome()
        for gr in grs:
            cov = CoverageSet("chrom",gr)
            bwpath = os.path.join(phastCons46way_dir, gr.sequences[0].chrom+".phastCons46way.bw")
            cov.coverage_from_bigwig(bigwig_file=bwpath,stepsize=stepsize)
            self.coverage += cov.coverage

    def count_unique_reads(self, bamFile):
        """Count the number of unique reads on for class variable <genomicRegions>.
        
        *Keyword arguments:*
        
        - bamFile -- path to bigwig file
        
        *Output:*
        
        number of unique reads
        
        """
        
        bam = pysam.Samfile(bamFile, "rb" )

        reads = []
        for i,region in enumerate(self.genomicRegions):
            for r in bam.fetch(region.chrom,region.initial,region.final):
                reads.append(r.qname)

        reads = list(set(reads))
        return len(reads)



    def norm_gc_content(self, cov, genome_path, chrom_sizes):
        chrom_sizes_dict = {}

        with open(chrom_sizes) as f:
            for line in f:
                line = line.strip()
                line = line.split('\t')
                c, e = line[0], int(line[1])
                chrom_sizes_dict[c] = e

        gc_cov, gc_avg, _ = get_gc_context(self.stepsize, self.binsize, genome_path, cov, chrom_sizes_dict)

        import warnings  # todo: ugly, why do warnings occur?
        warnings.filterwarnings("ignore")

        for i in range(len(self.coverage)):
            assert len(self.coverage[i]) == len(gc_cov[i])
            self.coverage[i] = np.array(self.coverage[i])
            gc_cov[i] = np.array(gc_cov[i])
            gc_cov[i][gc_cov[i] < 10 * -300] = gc_avg  # sometimes zeros occur, do not consider
            self.coverage[i] = self.coverage[i] * gc_avg / gc_cov[i]
            self.coverage[i] = self.coverage[i].clip(0, max(max(self.coverage[i]), 0))  # neg. values to 0
            self.coverage[i] = self.coverage[i].astype(int)

def get_gc_context(stepsize, binsize, genome_path, cov_list, chrom_sizes_dict):
    """Get GC content"""
    class help_content():
        def __init__(self):
            self.content = [[] for _ in range(101)]
            self.g_gc = []
            self.g = -1

        def add(self, d, c):
            self.content[int(c * 100)].append(round(float(d), 2))

        def _compute(self):
            for l in self.content:
                r = sum(l) / float(len(l)) if len(l) > 0 else 0
                self.g_gc.append(r)

            self.g = sum(self.g_gc) / float(len(self.g_gc))

        def _map(self, x):
            return self.g_gc[x]

    # chromosomes = []
    # get first chromosome, typically chr1
    # for s in FastaReader(genome_path):
    #    chromosomes.append(s.name)
    tmp = chrom_sizes_dict.keys()
    # tmp = map(lambda x: x.replace('chr',''), chromosomes)
    tmp.sort()
    genome_fasta = pysam.Fastafile(genome_path)
    genome = genome_fasta.fetch(reference=tmp[0])

    content = help_content()
    gc_content_cov = []

    for cov in cov_list:
        cur_gc_value = []

        for i in range(len(cov)):
            s = i * stepsize
            e = i * stepsize + binsize
            seq = genome[s: e + 1]
            seq = seq.upper()
            count = seq.count("C") + seq.count("G")
            if len(seq) > 0:
                gc_content = float(count) / len(seq)
                # print(float(count), len(seq), float(count) / len(seq), cov[i], file=sys.stderr)
                content.add(cov[i], gc_content)
                cur_gc_value.append(int(gc_content * 100))
            else:
                #                print("Bins exceed genome (%s - %s), add pseudo counts" %(s, e), file=sys.stderr)
                #                     content.add(cov[i], 0.5)
                cur_gc_value.append(0)

        gc_content_cov.append(cur_gc_value)

    content._compute()
    r = []
    for l in gc_content_cov:
        tmp = map(content._map, l)
        r.append(tmp)

    return r, content.g, content.g_gc



###################################################################################################
##### Annotation File Standard (tab-separated):
# MATRIX_ID: The matrices' ID. It may change format for different repositories. (STRING)
# PWM_NAME: Name of the PWM inside the respective repository. (STRING - without the .pwm)
# SOURCE: The source repository of such matrix. (STRING)
# VERSION: The version of such matrix (1 for 'primary motif', 2 for 'secondary motif', etc). (INT)
# GENE_NAMES: Name of genes associated with such TF matrix. (LIST)
# GROUP: <To be used in future - Will represent clusters of motifs>. (STRING)
###################################################################################################
# * Mandatory fields: MATRIX_ID, SOURCE, VERSION, GENE_NAMES.
# * Fields with multiple entries should be separated by ';' (no spaces).
# * Fields with missing/non-existing/doesn't matter data should be filled with '.'
# * Co-binding should be represented by '+' (no spaces).
###################################################################################################

# Import
import os
import sys
import glob

# Parameters
dataLocation = "./"
group = "."

###################################################################################################
# HOCOMOCO
###################################################################################################

# Fetching file names
# TODO: check if this still works for hocomoco v10
source = "hocomoco"
inputLocation = dataLocation+source+"/"
resultMatrix = []
for inputFileName in glob.glob(inputLocation+"*.pwm"):
    ll = inputFileName.split("/")[-1].split(".")[0].split("_")
    matrix_id = ll[1]
    pwm_name = ".".join(inputFileName.split("/")[-1].split(".")[:-1])
    version = "1"
    gene_names = ll[0]
    resultMatrix.append([matrix_id,pwm_name,source,version,gene_names,group])

# Sorting results by ID
resultMatrix = sorted(resultMatrix ,key=lambda x: x[2])
resultMatrix = sorted(resultMatrix ,key=lambda x: x[0])

# Writing to output file
outputFileName = dataLocation+source+".mtf"
outputFile = open(outputFileName,"w")
for resultVec in resultMatrix:
    outputFile.write("\t".join(resultVec)+"\n")
outputFile.close()

###################################################################################################
# JASPAR VERTEBRATES
###################################################################################################

# Fetching file names
source = "jaspar_vertebrates"
inputLocation = dataLocation+source+"/"
resultMatrix = []
for inputFileName in glob.glob(inputLocation+"*.pwm"):
    ll = inputFileName.split("/")[-1].split(".")
    matrix_id = ll[0]
    pwm_name = ".".join(inputFileName.split("/")[-1].split(".")[:-1])
    version = "1"
    if(len(ll) > 4): version = ll[4]
    gene_names = ll[2].replace("::","+")
    resultMatrix.append([matrix_id,pwm_name,source,version,gene_names,group])

# Sorting results by ID
resultMatrix = sorted(resultMatrix ,key=lambda x: x[2])
resultMatrix = sorted(resultMatrix ,key=lambda x: x[0])

# Writing to output file
outputFileName = dataLocation+source+".mtf"
outputFile = open(outputFileName,"w")
for resultVec in resultMatrix:
    outputFile.write("\t".join(resultVec)+"\n")
outputFile.close()

###################################################################################################
# TRANSFAC PUBLIC
###################################################################################################

# Fetching file names
source = "transfac_public"
inputLocation = dataLocation+source+"/"
resultMatrix = []
for inputFileName in glob.glob(inputLocation+"*.pwm"):
    ll = inputFileName.split("/")[-1].split(".")[0].split("_")
    matrix_id = ll[0]
    pwm_name = ".".join(inputFileName.split("/")[-1].split(".")[:-1])
    version = "1"
    gene_names = ll[1]
    resultMatrix.append([matrix_id,pwm_name,source,version,gene_names,group])

# Sorting results by ID
resultMatrix = sorted(resultMatrix ,key=lambda x: x[2])
resultMatrix = sorted(resultMatrix ,key=lambda x: x[0])

# Writing to output file
outputFileName = dataLocation+source+".mtf"
outputFile = open(outputFileName,"w")
for resultVec in resultMatrix:
    outputFile.write("\t".join(resultVec)+"\n")
outputFile.close()

###################################################################################################
# UNIPROBE PRIMARY
###################################################################################################

# Fetching file names
source = "uniprobe_primary"
inputLocation = dataLocation+source+"/"
resultMatrix = []
for inputFileName in glob.glob(inputLocation+"*.pwm"):
    ll = inputFileName.split("/")[-1].split(".")[0].split("_")
    matrix_id = ll[0]
    pwm_name = ".".join(inputFileName.split("/")[-1].split(".")[:-1])
    version = ll[1]
    gene_names = ll[2]
    resultMatrix.append([matrix_id,pwm_name,source,version,gene_names,group])

# Sorting results by ID
resultMatrix = sorted(resultMatrix ,key=lambda x: x[2])
resultMatrix = sorted(resultMatrix ,key=lambda x: x[0])

# Writing to output file
outputFileName = dataLocation+source+".mtf"
outputFile = open(outputFileName,"w")
for resultVec in resultMatrix:
    outputFile.write("\t".join(resultVec)+"\n")
outputFile.close()

###################################################################################################
# UNIPROBE SECONDARY
###################################################################################################

# Fetching file names
source = "uniprobe_secondary"
inputLocation = dataLocation+source+"/"
resultMatrix = []
for inputFileName in glob.glob(inputLocation+"*.pwm"):
    ll = inputFileName.split("/")[-1].split(".")[0].split("_")
    matrix_id = ll[0]
    pwm_name = ".".join(inputFileName.split("/")[-1].split(".")[:-1])
    version = ll[1]
    gene_names = ll[2]
    resultMatrix.append([matrix_id,pwm_name,source,version,gene_names,group])

# Sorting results by ID and version
resultMatrix = sorted(resultMatrix ,key=lambda x: x[2])
resultMatrix = sorted(resultMatrix ,key=lambda x: x[0])

# Writing to output file
outputFileName = dataLocation+source+".mtf"
outputFile = open(outputFileName,"w")
for resultVec in resultMatrix:
    outputFile.write("\t".join(resultVec)+"\n")
outputFile.close()





# Python Libraries
from __future__ import print_function
from __future__ import division
import time
import numpy
import matplotlib.pyplot as plt

# Local Libraries
# Distal Libraries
from rgt.Util import Html
from rgt.CoverageSet import *
from rgt.ExperimentalMatrix import *
from shared_function import gen_tags, tag_from_r, print2, MyPool, compute_coverage, colormap, unique, output_array
# Local test
dir = os.getcwd()

###########################################################################################
#                    Lineplot
###########################################################################################


class Lineplot:
    def __init__(self, EMpath, title, annotation, organism, center, extend, rs, bs, ss, df, dft, fields, test, sense):

        # Read the Experimental Matrix
        self.title = title
        self.exps = ExperimentalMatrix()
        self.exps.read(EMpath, test=test)
        for f in self.exps.fields:
            if f not in ['name', 'type', 'file', "reads", "regions", "factors"]:
                self.exps.match_ms_tags(f, test=test)
                self.exps.remove_name()

        # if annotation:
        #     self.beds, self.bednames, self.annotation = annotation_dump(organism)

        # else:
        self.beds = self.exps.get_regionsets()  # A list of GenomicRegionSets
        self.bednames = self.exps.get_regionsnames()
        self.annotation = None

        self.reads = self.exps.get_readsfiles()
        self.readsnames = self.exps.get_readsnames()
        self.fieldsDict = self.exps.fieldsDict
        self.parameter = []
        self.center = center
        self.extend = extend
        self.rs = rs
        self.bs = bs
        self.ss = ss
        self.df = df
        self.dft = dft
        self.sense = sense

    def relocate_bed(self):
        self.processed_beds = []
        self.processed_bedsF = []  # Processed beds to be flapped

        for bed in self.beds:
            if self.center == 'bothends':
                newbed = bed.relocate_regions(center='leftend',
                                              left_length=self.extend + self.bs,
                                              right_length=self.extend + self.bs)
                self.processed_beds.append(newbed)
                newbedF = bed.relocate_regions(center='rightend',
                                               left_length=self.extend + self.bs,
                                               right_length=self.extend + self.bs)
                self.processed_bedsF.append(newbedF)
            elif self.center == 'upstream' or self.center == 'downstream':
                allbed = bed.relocate_regions(center=self.center,
                                              left_length=self.extend + self.bs,
                                              right_length=self.extend + self.bs)
                newbed = allbed.filter_strand(strand="+")
                self.processed_beds.append(newbed)
                newbedF = allbed.filter_strand(strand="-")
                self.processed_bedsF.append(newbedF)
            else:
                newbed = bed.relocate_regions(center=self.center,
                                              left_length=self.extend + int(0.5 * self.bs) + 2 * self.ss,
                                              right_length=self.extend + int(0.5 * self.bs) + 2 * self.ss)
                self.processed_beds.append(newbed)

    def group_tags(self, groupby, sortby, colorby):
        """Generate the tags for the grouping of plot
        Parameters:
            groupby = 'reads','regions','cell',or 'factor'
            colorby = 'reads','regions','cell',or 'factor'
            sortby = 'reads','regions','cell',or 'factor'
        """
        self.tag_type = [sortby, groupby, colorby, self.dft]
        if "None" in self.tag_type: self.tag_type.remove("None")

        if groupby == "None":
            self.group_tags = [""]
        elif groupby == "regions" and self.annotation:
            self.group_tags = self.bednames
        else:
            self.group_tags = gen_tags(self.exps, groupby)

        if sortby == "None":
            self.sort_tags = [""]
        elif sortby == "regions" and self.annotation:
            self.sort_tags = self.bednames
        else:
            self.sort_tags = gen_tags(self.exps, sortby)

        if colorby == "None":
            self.color_tags = [""]
        elif colorby == "regions" and self.annotation:
            self.color_tags = self.bednames
        else:
            self.color_tags = gen_tags(self.exps, colorby)

        print("\tColumn labels:\t" + ",".join(self.group_tags))
        print("\tRow labels:\t" + ",".join(self.sort_tags))
        print("\tColor labels:\t" + ",".join(self.color_tags))

    def gen_cues(self):
        self.cuebed = OrderedDict()
        self.cuebam = OrderedDict()

        # if self.annotation:
        #     #all_tags = []
        #     #for dictt in self.exps.fieldsDict.values():
        #     #    for tag in dictt.keys():
        #     #        all_tags.append(tag)
        #     for bed in self.bednames:
        #     #    self.cuebed[bed] = set([bed]+all_tags)
        #         self.cuebed[bed] = set([bed])
        # else:
        for bed in self.bednames:
            self.cuebed[bed] = set(tag_from_r(self.exps, self.tag_type, bed))
            try:
                self.cuebed[bed].remove("None")
            except:
                pass
        for bam in self.readsnames:
            self.cuebam[bam] = set(tag_from_r(self.exps, self.tag_type, bam))

    def coverage(self, sortby, heatmap=False, logt=False, mp=0, log=False):

        def annot_ind(bednames, tags):
            """Find the index for annotation tag"""
            for ind, a in enumerate(bednames):
                if a in tags: return ind

        if mp>0: ts = time.time()
        normRPM = False
        # Calculate for coverage
        mp_input = []
        data = OrderedDict()

        bi = 0
        for s in self.sort_tags:
            data[s] = OrderedDict()
            for g in self.group_tags:
                data[s][g] = OrderedDict()
                for c in self.color_tags:
                    # if self.df: data[s][g][c] = []
                    data[s][g][c] = OrderedDict()
                    if not self.dft:
                        dfs = [c]
                    else:
                        dfs = self.exps.fieldsDict[self.dft].keys()
                    for d in dfs:
                        data[s][g][c][d] = defaultdict(list)
                        for bed in self.cuebed.keys():
                            # print(self.cuebed[bed])
                            # print(set([s,g,c,d]))
                            # print(self.cuebed[bed].issubset(set([s,g,c,d])))
                            if len(self.cuebed[bed].intersection(set([s, g, c, d]))) > 2 or self.cuebed[bed].issubset(
                                    set([s, g, c, d])):
                                # if self.cuebed[bed] <= set([s,g,c]):
                                for bam in self.cuebam.keys():

                                    # print(self.cuebam[bam])
                                    # print(set([s,g,c]))
                                    if self.cuebam[bam] <= set([s, g, c, d]):
                                        i = self.bednames.index(bed)
                                        j = self.readsnames.index(bam)
                                        # print(bed + "." + bam)

                                        # if len(self.processed_beds[i]) == 0:
                                        #     try:
                                        #         data[s][g][c][d].append(numpy.empty(1, dtype=object))
                                        #     except:
                                        #         data[s][g][c][d] = [numpy.empty(1, dtype=object)]
                                        #     continue
                                        #########################################################################
                                        if mp > 0:  # Multiple processing
                                            mp_input.append([self.processed_beds[i], self.reads[j],
                                                             self.rs, self.bs, self.ss, self.center, heatmap, logt,
                                                             s, g, c, d])
                                            data[s][g][c][d] = None

                                        #########################################################################
                                        else:  # Single thread
                                            ts = time.time()
                                            cov = CoverageSet(bed + "." + bam, self.processed_beds[i])

                                            # print(len(self.processed_beds[i]))
                                            if "Conservation" in [s,g,c,d]:
                                                cov.phastCons46way_score(stepsize=self.ss)

                                            elif ".bigwig" in self.reads[j].lower() or ".bw" in self.reads[j].lower():
                                                cov.coverage_from_bigwig(bigwig_file=self.reads[j], stepsize=self.ss)
                                            else:
                                                if not self.sense:
                                                    cov.coverage_from_bam(bam_file=self.reads[j],
                                                                          extension_size=self.rs, binsize=self.bs,
                                                                          stepsize=self.ss)
                                                    if normRPM: cov.normRPM()
                                                else:  # Sense specific
                                                    cov.coverage_from_bam(bam_file=self.reads[j],
                                                                          extension_size=self.rs, binsize=self.bs,
                                                                          stepsize=self.ss, get_sense_info=True,
                                                                          paired_reads=True)
                                                    cov.array_transpose()
                                                    if normRPM: cov.normRPM()

                                            # When bothends, consider the fliping end
                                            if self.center == 'bothends' or self.center == 'upstream' or self.center == 'downstream':
                                                if "Conservation" in [s,g,c,d]:
                                                    flap = CoverageSet("for flap", self.processed_bedsF[i])
                                                    flap.phastCons46way_score(stepsize=self.ss)
                                                    ffcoverage = numpy.fliplr(flap.coverage)
                                                    cov.coverage = numpy.concatenate((cov.coverage, ffcoverage), axis=0)
                                                elif ".bigwig" in self.reads[j].lower() or ".bw" in self.reads[j].lower():
                                                    flap = CoverageSet("for flap", self.processed_bedsF[i])
                                                    flap.coverage_from_bigwig(bigwig_file=self.reads[j],
                                                                              stepsize=self.ss)
                                                    ffcoverage = numpy.fliplr(flap.coverage)
                                                    cov.coverage = numpy.concatenate((cov.coverage, ffcoverage), axis=0)
                                                else:
                                                    flap = CoverageSet("for flap", self.processed_bedsF[i])
                                                    if not self.sense:
                                                        flap.coverage_from_bam(self.reads[j], extension_size=self.rs,
                                                                               binsize=self.bs, stepsize=self.ss)
                                                        if normRPM: flap.normRPM()
                                                    else:  # Sense specific
                                                        flap.coverage_from_bam(bam_file=self.reads[j],
                                                                               extension_size=self.rs, binsize=self.bs,
                                                                               stepsize=self.ss, get_sense_info=True,
                                                                               paired_reads=True)
                                                        flap.array_transpose(flip=True)
                                                        if normRPM: flap.normRPM()
                                                    ffcoverage = numpy.fliplr(flap.coverage)
                                                    try: cov.coverage = numpy.concatenate((cov.coverage, ffcoverage), axis=0)
                                                    except: pass

                                                    if self.sense:
                                                        cov.transpose_cov1 = numpy.concatenate((cov.transpose_cov1,
                                                                                                flap.transpose_cov1),axis=0)
                                                        cov.transpose_cov2 = numpy.concatenate((cov.transpose_cov2,
                                                                                                flap.transpose_cov2), axis=0)

                                            # Averaging the coverage of all regions of each bed file
                                            if heatmap:
                                                if logt:
                                                    data[s][g][c][d] = numpy.log10(numpy.vstack(
                                                        cov.coverage) + 1)  # Store the array into data list
                                                else:
                                                    data[s][g][c][d] = numpy.vstack(
                                                        cov.coverage)  # Store the array into data list
                                            else:
                                                if len(cov.coverage) == 0:
                                                    data[s][g][c][d] = None
                                                    print("** Warning: Cannot open " + self.reads[j])
                                                    continue
                                                else:
                                                    for i, car in enumerate(cov.coverage):
                                                        if i == 0: avearr = np.array(car, ndmin=2)
                                                        else:
                                                            # avearr = numpy.vstack((avearr, np.array(car, ndmin=2)))
                                                            try: avearr = numpy.vstack((avearr, np.array(car, ndmin=2)))
                                                            except: print(bed+"."+bam+"."+str(i))
                                                    if log:
                                                        avearr = numpy.log2(avearr+1)

                                                    avearr = numpy.average(avearr, axis=0)
                                                    if self.sense:
                                                        if log:
                                                            sense_1 = numpy.average(numpy.log2(cov.transpose_cov1+1), axis=0)
                                                            sense_2 = numpy.average(numpy.log2(cov.transpose_cov2+1), axis=0)
                                                        else:
                                                            sense_1 = numpy.average(cov.transpose_cov1,axis=0)
                                                            sense_2 = numpy.average(cov.transpose_cov2,axis=0)
                                                    cut_end = int(self.bs/self.ss)
                                                    avearr = avearr[cut_end:-cut_end]
                                                    data[s][g][c][d]["all"].append(avearr)

                                                    if self.sense:
                                                        sense_1 = sense_1[cut_end:-cut_end]
                                                        sense_2 = sense_2[cut_end:-cut_end]
                                                        data[s][g][c][d]["sense_1"].append(sense_1)
                                                        data[s][g][c][d]["sense_2"].append(sense_2)

                                            bi += 1
                                            te = time.time()
                                            print2(self.parameter,
                                                   "\t" + str(bi) + "\t" + "{0:30}\t--{1:<5.1f}s".format(
                                                       bed + "." + bam, ts - te))


        if mp > 0:
            pool = MyPool(mp)
            mp_output = pool.map(compute_coverage, mp_input)
            pool.close()
            pool.join()
            for s in data.keys():
                for g in data[s].keys():
                    for c in data[s][g].keys():
                        for d in data[s][g][c].keys():
                            for out in mp_output:
                                if out[0] == s and out[1] == g and out[2] == c and out[3] == d:
                                    if self.df:
                                        try:
                                            data[s][g][c][d][-1].append(out[4])
                                        except:
                                            data[s][g][c][d] = [[out[4]]]
                                    else:
                                        try:
                                            data[s][g][c][d].append(out[4])
                                        except:
                                            data[s][g][c][d] = [out[4]]
        if self.df:
            for s in data.keys():
                for g in data[s].keys():
                    for c in data[s][g].keys():
                        for d in data[s][g][c].keys():
                            if isinstance(data[s][g][c][d]["all"], list) and len(data[s][g][c][d]["all"]) > 1:
                                diff = numpy.subtract(data[s][g][c][d]["all"][0], data[s][g][c][d]["all"][1])
                                data[s][g][c][d]["df"].append(diff.tolist())
                            else:
                                print("Warning: There is no repetitive reads for calculating difference.\n"
                                      "         Please add one more entry in experimental matrix.")
        self.data = data

    def colormap(self, colorby, definedinEM):
        colors = colormap(self.exps, colorby, definedinEM, annotation=self.annotation)
        self.colors = {}
        for i, c in enumerate(self.color_tags):
            self.colors[c] = colors[i]

    def plot(self, groupby, colorby, output, printtable=False, scol=False, srow=False, w=2, h=2):

        rot = 50
        if len(self.data.values()[0].keys()) < 2:
            ticklabelsize = w * 1.5
        else:
            ticklabelsize = w * 3
        tw = len(self.data.values()[0].keys()) * w
        th = len(self.data.keys()) * (h * 0.8)

        f, axs = plt.subplots(len(self.data.keys()), len(self.data.values()[0].keys()), dpi=300,
                              figsize=(tw, th))

        yaxmax = [0] * len(self.data.values()[0])
        sx_ymax = [0] * len(self.data.keys())
        if self.df:
            yaxmin = [0] * len(self.data.values()[0])
            sx_ymin = [0] * len(self.data.keys())

        if printtable:
            bott = self.extend - int(0.5 * self.ss)
            pArr = [["Group_tag", "Sort_tag", "Color_tag"] + [str(x) for x in range(-bott, bott, self.ss)]]  # Header
        nit = len(self.data.keys())
        for it, s in enumerate(self.data.keys()):

            for i, g in enumerate(self.data[s].keys()):
                try:
                    ax = axs[it, i]
                except:
                    if len(self.data.keys()) == 1 and len(self.data[s].keys()) == 1:
                        ax = axs
                    elif len(self.data.keys()) == 1 and len(self.data[s].keys()) > 1:
                        ax = axs[i]
                    else:
                        ax = axs[it]

                if it == 0:
                    if self.df:
                        ax.set_title(g + "_df", fontsize=ticklabelsize + 2)
                    else:
                        ax.set_title(g, fontsize=ticklabelsize + 2)

                # Processing for future output
                for j, c in enumerate(self.data[s][g].keys()):

                    for k, d in enumerate(self.data[s][g][c].keys()):
                        if not self.data[s][g][c][d]:
                            continue
                        else:
                            if not self.sense:
                                if self.df: pt = self.data[s][g][c][d]["df"]
                                else: pt = self.data[s][g][c][d]["all"]

                                for l, y in enumerate(pt):
                                    # print(y)
                                    yaxmax[i] = max(numpy.amax(y), yaxmax[i])
                                    sx_ymax[it] = max(numpy.amax(y), sx_ymax[it])
                                    if self.df:
                                        yaxmin[i] = min(numpy.amin(y), yaxmin[i])
                                        sx_ymin[it] = min(numpy.amin(y), sx_ymin[it])

                                    x = numpy.linspace(-self.extend, self.extend, len(y))
                                    ax.plot(x, y, color=self.colors[c], lw=1, label=c)
                                    if it < nit - 1:
                                        ax.set_xticklabels([])
                                    # Processing for future output
                                    if printtable: pArr.append([g, s, c, d] + list(y))
                            else:
                                plt.text(0.5, 0.51, 'sense',transform=ax.transAxes,fontsize=ticklabelsize,
                                         horizontalalignment='center', verticalalignment='bottom')
                                plt.text(0.5, 0.49, 'anti-sense', transform=ax.transAxes,fontsize=ticklabelsize,
                                         horizontalalignment='center', verticalalignment='top')
                                plt.plot((-self.extend, self.extend), (0, 0), '0.1', linewidth=0.2)
                                print(self.data[s][g][c][d])
                                for l, y in enumerate(self.data[s][g][c][d]["sense_1"]):
                                    # print(y)
                                    ymax1 = numpy.amax(y)
                                    yaxmax[i] = max(ymax1, yaxmax[i])
                                    sx_ymax[it] = max(ymax1, sx_ymax[it])
                                    x = numpy.linspace(-self.extend, self.extend, y.shape[0])
                                    ax.plot(x, y, color=self.colors[c], lw=1, label=c)
                                    if it < nit - 1: ax.set_xticklabels([])
                                    # Processing for future output
                                    if printtable: pArr.append([g, s, c, d, "+"] + list(y))

                                for l, y in enumerate(self.data[s][g][c][d]["sense_2"]):
                                    # print(y)
                                    ymax2 = numpy.amax(y)
                                    yaxmax[i] = max(ymax2, yaxmax[i])
                                    sx_ymax[it] = max(ymax2, sx_ymax[it])
                                    x = numpy.linspace(-self.extend, self.extend, y.shape[0])
                                    ax.plot(x, -y, color=self.colors[c], lw=1, label=c)
                                    if it < nit - 1: ax.set_xticklabels([])
                                    # Processing for future output
                                    if printtable: pArr.append([g, s, c, d, "-"] + list(y))
                                ym = 1.2 * max(max(yaxmax), max(sx_ymax))
                                ax.set_ylim([-ym, ym])

                ax.get_yaxis().set_label_coords(-0.1, 0.5)
                ax.set_xlim([-self.extend, self.extend])
                plt.setp(ax.get_xticklabels(), fontsize=ticklabelsize, rotation=rot)
                plt.setp(ax.get_yticklabels(), fontsize=ticklabelsize)


                ax.locator_params(axis='x', nbins=4)
                ax.locator_params(axis='y', nbins=2)
                # try:
                #
                # except:
                #     ax.locator_params(axis='y', nbins=2)
                #     pass
        if printtable:
            output_array(pArr, directory=output, folder=self.title, filename="plot_table.txt")

        for it, ty in enumerate(self.data.keys()):
            try:
                axs[it, 0].set_ylabel("{}".format(ty), fontsize=ticklabelsize + 1)
            except:
                try:
                    axs[it].set_ylabel("{}".format(ty), fontsize=ticklabelsize + 1)
                except:
                    axs.set_ylabel("{}".format(ty), fontsize=ticklabelsize + 1)

            for i, g in enumerate(self.data[ty].keys()):
                try: axx = axs[it, i]
                except:
                    try:
                        if len(self.data.keys()) == 1:
                            axx = axs[i]
                        else:
                            axx = axs[it]
                    except: axx = axs

                if self.df:
                    if scol:
                        ymin = yaxmin[i] - abs(yaxmin[i] * 0.2)
                        ymax = yaxmax[i] + abs(yaxmax[i] * 0.2)
                    elif srow:
                        ymin = sx_ymin[it] - abs(sx_ymin[it] * 0.2)
                        ymax = sx_ymax[it] + abs(sx_ymax[it] * 0.2)

                else:
                    if scol: ymax = yaxmax[i] * 1.2
                    elif srow: ymax = sx_ymax[it] * 1.2
                    else:
                        ymax = axx.get_ylim()[1]
                    if self.sense: ymin = -ymax
                    else: ymin = 0

                try: axx.set_ylim([ymin, ymax])
                except: pass

        handles, labels = ax.get_legend_handles_labels()
        uniq_labels = unique(labels)

        plt.legend([handles[labels.index(l)] for l in uniq_labels], uniq_labels, loc='center left', handlelength=1,
                   handletextpad=1,
                   columnspacing=2, borderaxespad=0., prop={'size': ticklabelsize}, bbox_to_anchor=(1.05, 0.5))

        f.tight_layout()
        self.fig = f

    def gen_html(self, directory, title, align=50):
        dir_name = os.path.basename(directory)
        # check_dir(directory)
        html_header = dir_name + " / " + title
        link_d = OrderedDict()
        link_d["Lineplot"] = "index.html"
        link_d["Parameters"] = "parameters.html"

        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")
        html.add_figure("lineplot.png", align="center", width="80%")

        html.write(os.path.join(directory, title, "index.html"))

        ## Parameters
        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")
        type_list = 'ssssssssss'
        col_size_list = [20, 20, 20, 20, 20, 20, 20, 20, 20]
        header_list = ["Assumptions and hypothesis"]
        data_table = []
        if self.annotation:
            data_table.append(
                ["Genomic annotation: TSS - Transcription Start Site; TTS - Transcription Termination Site."])
        data_table.append(["Directory:      " + directory.rpartition("/")[2]])
        data_table.append(["Title:          " + title])
        data_table.append(["Extend length:  " + str(self.extend)])
        data_table.append(["Read size:      " + str(self.rs)])
        data_table.append(["Bin size:       " + str(self.bs)])
        data_table.append(["Step size:      " + str(self.ss)])
        data_table.append(["Center mode:    " + self.center])

        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align,
                             cell_align="left")

        html.add_free_content(['<a href="parameters.txt" style="margin-left:100">See parameters</a>'])
        html.add_free_content(['<a href="experimental_matrix.txt" style="margin-left:100">See experimental matrix</a>'])

        html.write(os.path.join(directory, title, "parameters.html"))

    def hmsort(self, sort):
        if sort == None:
            pass
        elif sort == 0:
            for t in self.data.keys():
                for i, g in enumerate(self.data[t].keys()):
                    # print(numpy.sum(data[t][bed].values()[0], axis=1))
                    # print(len(numpy.sum(data[t][bed].values()[0], axis=1)))

                    sumarr = numpy.sum([numpy.sum(d, axis=1) for d in self.data[t][g].values()], axis=0)
                    # print(sumarr)
                    # sumarr = numpy.sum(sumarr, axis=1)
                    ind = stats.rankdata(sumarr, method='ordinal')  # The index for further sorting
                    # numpy.fliplr(ind)

                    for j, c in enumerate(self.data[t][g].keys()):
                        d = numpy.empty(shape=(self.data[t][g][c].shape))
                        for k, ranki in enumerate(ind):
                            d[-ranki, :] = self.data[t][g][c][k, :]
                        self.data[t][g][c] = d
        else:
            for t in self.data.keys():
                for i, g in enumerate(self.data[t].keys()):
                    sumarr = numpy.sum(self.data[t][g].values()[sort - 1], axis=1)
                    # print(sumarr)
                    # sumarr = numpy.sum(sumarr, axis=1)
                    ind = stats.rankdata(sumarr, method='ordinal')  # The index for further sorting
                    # list(ind)
                    # print(ind)
                    for j, c in enumerate(self.data[t][g].keys()):
                        d = numpy.empty(shape=(self.data[t][g][c].shape))
                        for k, ranki in enumerate(ind):
                            d[-ranki, :] = self.data[t][g][c][k, :]
                        self.data[t][g][c] = d
                        # print(data[t][bed].values()[0])

    def hmcmlist(self, colorby, definedinEM):
        # self.colors = colormaps(self.exps, colorby, definedinEM)
        self.colors = ["Reds", "Blues", "Oranges", "Greens", "Purples"]

    def heatmap(self, logt):
        tickfontsize = 6
        ratio = 10
        self.hmfiles = []
        self.figs = []
        for ti, t in enumerate(self.data.keys()):
            # fig.append(plt.figure())
            # rows = len(data[t].keys())
            columns = len(self.data[t].values()[0].keys())
            # fig, axs = plt.subplots(rows,columns, sharey=True, dpi=300)
            # matplotlib.pyplot.subplots_adjust(left=1, right=2, top=2, bottom=1)
            fig = plt.figure(t)
            plt.suptitle("Heatmap: " + t, y=1.05)
            rows = len(self.data[t].keys())

            # gs = gridspec.GridSpec(rows*ratio,columns)
            axs = numpy.empty(shape=(rows + 1, columns), dtype=object)

            for bi, g in enumerate(self.data[t].keys()):
                for bj, c in enumerate(self.data[t][g].keys()):
                    max_value = numpy.amax(self.data[t][g][c])
                    max_value = int(max_value)
                    axs[bi, bj] = plt.subplot2grid(shape=(rows * ratio + 1, columns), loc=(bi * ratio, bj),
                                                   rowspan=ratio)
                    if bi == 0: axs[bi, bj].set_title(c, fontsize=7)
                    # print(self.data[t][g][c])
                    # print(self.colors)
                    # print(bj)
                    # im = axs[bi, bj].imshow(self.data[t][g][c], extent=[-self.extend, self.extend, 0,1], aspect='auto',
                    #                        vmin=0, vmax=max_value, interpolation='nearest', cmap=self.colors[bj])

                    im = axs[bi, bj].imshow(self.data[t][g][c], extent=[-self.extend, self.extend, 0, 1], aspect='auto',
                                            vmin=0, vmax=max_value, interpolation='nearest', cmap=plt.get_cmap("Blues"))

                    # for bi, g in enumerate(self.data[t].keys()):
                    #    for bj, c in enumerate(self.data[t][g].keys()):


                    # im = axs[bi, bj].imshow(self.data[t][g][c], extent=[-self.extend, self.extend, 0,1], aspect='auto',
                    #                        vmin=0, vmax=max_value, interpolation='nearest', cmap=cm.coolwarm)
                    axs[bi, bj].set_xlim([-self.extend, self.extend])
                    axs[bi, bj].set_xticks([-self.extend, 0, self.extend])
                    # axs[bi, bj].set_xticklabels([-args.e, 0, args.e]
                    plt.setp(axs[bi, bj].get_xticklabels(), fontsize=tickfontsize, rotation=0)
                    # plt.setp(axs[bi, bj].get_yticklabels(), fontsize=10)
                    # axs[bi, bj].locator_params(axis = 'x', nbins = 2)
                    # axs[bi, bj].locator_params(axis = 'y', nbins = 4)
                    for spine in ['top', 'right', 'left', 'bottom']:
                        axs[bi, bj].spines[spine].set_visible(False)
                    axs[bi, bj].tick_params(axis='x', which='both', bottom='off', top='off', labelbottom='on')
                    axs[bi, bj].tick_params(axis='y', which='both', left='off', right='off', labelleft='off')

                    # if bj > 0:
                    #    plt.setp(axs[bi, bj].get_yticklabels(),visible=False)
                    # plt.setp(axarr[i].get_yticks(),visible=False)
                    axs[bi, bj].minorticks_off()
                    if bj == 0:
                        # nregion = len(self.exps.objectsDict[g])
                        # axs[bi, bj].set_ylabel(self.exps.get_type(g,'factor')+" ("+str(nregion) + ")", fontsize=7)
                        axs[bi, bj].set_ylabel(g, fontsize=7)
                    if bi == rows - 1:
                        # divider = make_axes_locatable(axs[bi,bj])
                        # cax = divider.append_axes("bottom", size="5%", pad=0.5)
                        cbar_ax = plt.subplot2grid((rows * ratio + 4, columns), (rows * ratio + 3, bj))
                        # axs[rows,bj].tick_params(axis='y', which='both', left='off', right='off', labelleft='off')


                        # cbar = grid.cbar_axes[i//2].colorbar(im)
                        # cbar = plt.colorbar(im, cax = axs[rows,bj], ticks=[0, max_value], orientation='horizontal')
                        # cbar = axs[rows,bj].imshow(range(int(max_value)), extent=[0, int(max_value),0,0], aspect=10, extent=[-self.extend, self.extend,0,0]
                        #                           vmin=0, vmax=max_value, interpolation='nearest', cmap=self.colors[bj])
                        # cbar = axs[rows,bj].imshow(self.data[t][g][c], extent=[-self.extend, self.extend, 0,1], aspect='auto',
                        #                    vmin=0, vmax=max_value, interpolation='nearest', cmap=self.colors[bj])
                        # cbar = axs[rows,bj].imshow([range(2*self.extend),range(2*self.extend),range(2*self.extend)],
                        #                           aspect='auto', vmin=0, vmax=max_value, interpolation='nearest', cmap=self.colors[bj] )
                        # cbar.outline.set_linewidth(0.5)
                        # axs[rows,bj].set_ticks_position('none')
                        # axs[rows,bj].tick_params(axis='x', which='both', bottom='off', top='off', labelbottom='off')
                        # axs[rows,bj].tick_params(axis='y', which='both', left='off', right='off', labelleft='off')


                        # cbar.set_label('Amplitute of signal')
                        max_value = int(max_value)
                        # width = 0.4/rows
                        # cbar_ax = fig.add_axes([0.01 + bj/columns, 0, width, 0.01])
                        cbar = plt.colorbar(im, cax=cbar_ax, ticks=[0, max_value], orientation='horizontal')
                        cbar.ax.set_xticklabels([0, int(max_value)])
                        if logt:
                            cbar.ax.set_xticklabels(['0', '{:1.1f}'.format(max_value)],
                                                    fontsize=tickfontsize)  # horizontal colorbar
                            cbar.set_label('log10', fontsize=tickfontsize)
                            # else:
                            # cbar.ax.set_xticklabels(['0', int(max_value)], fontsize=tickfontsize)# horizontal colorbar
                            # pass
                            # cbar.outline.set_linewidth(0.1)

            # fig.tight_layout()
            # fig.tight_layout(pad=1.08, h_pad=None, w_pad=None)
            # fig.tight_layout(pad=1, h_pad=1, w_pad=1)
            self.figs.append(fig)
            self.hmfiles.append("heatmap" + "_" + t)

    def gen_htmlhm(self, outputname, title, align=50):
        dir_name = os.path.basename(outputname)
        # check_dir(directory)
        html_header = title
        link_d = OrderedDict()
        link_d["Lineplot"] = "index.html"
        link_d["Parameters"] = "parameters.html"

        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")

        # Each row is a plot with its data
        for name in self.hmfiles:
            html.add_figure(name + ".png", align="center")
        html.write(os.path.join(outputname, title, "index.html"))

        ## Parameters
        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")

        html.add_free_content(['<a href="parameters.txt" style="margin-left:100">See parameters</a>'])
        html.add_free_content(['<a href="experimental_matrix.txt" style="margin-left:100">See experimental matrix</a>'])
        html.write(os.path.join(outputname, title, "parameters.html"))




###################################################################################################
# Setup Logo Data Script
#   This script will create all motif logos. Be aware that this is a time consuming procedure
#   depending on the total number of PWMs.
# Dependencies:
#   Biopython -> http://biopython.org/wiki/Main_Page
###################################################################################################

# Import
from optparse import OptionParser
from os import path, walk, mkdir
from sys import platform

from Bio import motifs

# Optional Input
usage_message = "python setupLogoData.py [options]"

# Initializing Option Parser
parser = OptionParser(usage=usage_message)

# Parameter: RGT Data Location
parser.add_option("--all", dest="all", action="store_true", default=False,
                  help="Fetch all data sets.")
parser.add_option("--hocomoco", dest="hocomoco", action="store_true", default=False,
                  help="Creates logos only for HOCOMOCO repository.")
parser.add_option("--jaspar-vertebrates", dest="jaspar_vertebrates", action="store_true", default=False,
                  help="Creates logos only for Jaspar Vertebrates repository.")
parser.add_option("--uniprobe-primary", dest="uniprobe_primary", action="store_true", default=False,
                  help="Creates logos only for Uniprobe (primary motifs) repository.")
parser.add_option("--uniprobe-secondary", dest="uniprobe_secondary", action="store_true", default=False,
                  help="Creates logos only for Uniprobe (secondary motifs) repository.")
options, arguments = parser.parse_args()
if options.all:
    options.hocomoco = True
    options.jaspar_vertebrates = True
    options.uniprobe_primary = True
    options.uniprobe_secondary = True

###################################################################################################
# Parameters
###################################################################################################

# Current rgt data path
curr_dir = path.dirname(path.realpath(__file__))

# Platform
supported_platforms = ["linux", "linux2", "darwin"]
if platform not in supported_platforms:
    print("ERROR: This package currently supports only unix-based systems (Linux and MAC OS X).")
    exit(0)

###################################################################################################
# Logo Graphs
###################################################################################################

# Creating logos
output_logos_dir = path.join(curr_dir, "logos")
if not path.exists(output_logos_dir): mkdir(output_logos_dir)
for dir_name, subdir_list, file_list in walk(path.join(curr_dir, "motifs")):
    base_name = path.basename(dir_name)
    if ((options.hocomoco and base_name == "hocomoco") or
            (options.jaspar_vertebrates and base_name == "jaspar_vertebrates") or
            (options.uniprobe_primary and base_name == "uniprobe_primary") or
            (options.uniprobe_secondary and base_name == "uniprobe_secondary")):
        output_dir = path.join(curr_dir, "logos", base_name)
        if not path.exists(output_dir):
            mkdir(output_dir)
        else:
            continue
        print "Creating logos for " + base_name
        for pwm_file_name in file_list:
            pwm_full_file_name = path.join(dir_name, pwm_file_name)
            if pwm_file_name.split(".")[-1] != "pwm": continue
            pwm_file = open(pwm_full_file_name, "r")
            logo_file_name = path.join(output_dir, ".".join(pwm_file_name.split(".")[:-1]) + ".png")
            pwm = motifs.read(pwm_file, "pfm")
            pwm.weblogo(logo_file_name, format="png_print", stack_width="medium", color_scheme="color_classic")
            pwm_file.close()
        print "OK"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
THOR detects differential peaks in multiple ChIP-seq profiles associated
with two distinct biological conditions.

Copyright (C) 2014-2016 Manuel Allhoff (allhoff@aices.rwth-aachen.de)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

@author: Manuel Allhoff
"""

# Python
from __future__ import print_function
import sys

# Internal
from dpc_help import get_peaks, _fit_mean_var_distr, initialize, merge_output, handle_input
from tracker import Tracker
from postprocessing import _output_BED, _output_narrowPeak
from rgt.THOR.neg_bin_rep_hmm import NegBinRepHMM, get_init_parameters, _get_pvalue_distr
from rgt.THOR.RegionGiver import RegionGiver
from rgt.THOR.postprocessing import filter_by_pvalue_strand_lag
from rgt import __version__

# External


TEST = False #enable to test THOR locally


def _write_info(tracker, report, **data):
    """Write information to tracker"""
    tracker.write(text=data['func_para'][0], header="Parameters for both estimated quadr. function y=max(|a|*x^2 + x + |c|, 0) (a)")
    tracker.write(text=data['func_para'][1], header="Parameters for both estimated quadr. function y=max(|a|*x^2 + x + |c|, 0) (c)")
    #tracker.write(text=data['init_mu'], header="Inital parameter estimate for HMM's Neg. Bin. Emission distribution (mu)")
    #tracker.write(text=data['init_alpha'], header="Inital parameter estimate for HMM's Neg. Bin. Emission distribution (alpha)")
    #tracker.write(text=data['m'].mu, header="Final HMM's Neg. Bin. Emission distribution (mu)")
    #tracker.write(text=data['m'].alpha, header="Final HMM's Neg. Bin. Emission distribution (alpha)")
    #tracker.write(text=data['m']._get_transmat(), header="Transmission matrix")
    
    if report:
        tracker.make_html()


def train_HMM(region_giver, options, bamfiles, genome, chrom_sizes, dims, inputs, tracker):
    """Train HMM"""
    
    while True:
        train_regions = region_giver.get_training_regionset()
        exp_data = initialize(name=options.name, dims=dims, genome_path=genome, regions=train_regions,
                              stepsize=options.stepsize, binsize=options.binsize, bamfiles=bamfiles,
                              exts=options.exts, inputs=inputs, exts_inputs=options.exts_inputs,
                              debug=options.debug, verbose=options.verbose, no_gc_content=options.no_gc_content,
                              factors_inputs=options.factors_inputs, chrom_sizes=chrom_sizes,
                              tracker=tracker, norm_regions=options.norm_regions,
                              scaling_factors_ip=options.scaling_factors_ip, save_wig=options.save_wig,
                              housekeeping_genes=options.housekeeping_genes, test=TEST, report=options.report,
                              chrom_sizes_dict=region_giver.get_chrom_dict(), end=True, counter=0, output_bw=False,
                              save_input=options.save_input, m_threshold=options.m_threshold,
                              a_threshold=options.a_threshold, rmdup=options.rmdup)
        if exp_data.count_positive_signal() > len(train_regions.sequences[0]) * 0.00001:
            tracker.write(text=" ".join(map(lambda x: str(x), exp_data.exts)), header="Extension size (rep1, rep2, input1, input2)")
            tracker.write(text=map(lambda x: str(x), exp_data.scaling_factors_ip), header="Scaling factors")
            break
    
    func, func_para = _fit_mean_var_distr(exp_data.overall_coverage, options.name, options.debug,
                                          verbose=options.verbose, outputdir=options.outputdir,
                                          report=options.report, poisson=options.poisson)
    exp_data.compute_putative_region_index()
     
    print('Compute HMM\'s training set', file=sys.stderr)
    training_set, s0, s1, s2 = exp_data.get_training_set(TEST, exp_data, options.name, options.foldchange,
                                                         options.threshold, options.size_ts, 3)
    init_alpha, init_mu = get_init_parameters(s0, s1, s2)
    m = NegBinRepHMM(alpha=init_alpha, mu=init_mu, dim_cond_1=dims[0], dim_cond_2=dims[1], func=func)
    training_set_obs = exp_data.get_observation(training_set)
     
    print('Train HMM', file=sys.stderr)
    m.fit([training_set_obs], options.hmm_free_para)
    distr = _get_pvalue_distr(m.mu, m.alpha, tracker)
         
    return m, exp_data, func_para, init_mu, init_alpha, distr


def run_HMM(region_giver, options, bamfiles, genome, chrom_sizes, dims, inputs, tracker, exp_data, m, distr):
    """Run trained HMM chromosome-wise on genomic signal and call differential peaks"""
    output, pvalues, ratios, no_bw_files = [], [], [], []
    print("Compute HMM's posterior probabilities and Viterbi path to call differential peaks", file=sys.stderr)
    
    for i, r in enumerate(region_giver):
        end = True if i == len(region_giver) - 1 else False
        print("- taking into account %s" % r.sequences[0].chrom, file=sys.stderr)
        
        exp_data = initialize(name=options.name, dims=dims, genome_path=genome, regions=r,
                              stepsize=options.stepsize, binsize=options.binsize,
                              bamfiles=bamfiles, exts=exp_data.exts, inputs=inputs,
                              exts_inputs=exp_data.exts_inputs, debug=options.debug,
                              verbose=False, no_gc_content=options.no_gc_content,
                              factors_inputs=exp_data.factors_inputs, chrom_sizes=chrom_sizes,
                              tracker=tracker, norm_regions=options.norm_regions,
                              scaling_factors_ip=exp_data.scaling_factors_ip, save_wig=options.save_wig,
                              housekeeping_genes=options.housekeeping_genes, test=TEST, report=False,
                              chrom_sizes_dict=region_giver.get_chrom_dict(), gc_content_cov=exp_data.gc_content_cov,
                              avg_gc_content=exp_data.avg_gc_content, gc_hist=exp_data.gc_hist,
                              end=end, counter=i, m_threshold=options.m_threshold, a_threshold=options.a_threshold,
                              rmdup=options.rmdup)
        if exp_data.no_data:
            continue
        
        no_bw_files.append(i)
        exp_data.compute_putative_region_index()
        
        if exp_data.indices_of_interest is None:
            continue
        
        states = m.predict(exp_data.get_observation(exp_data.indices_of_interest))
        
        inst_ratios, inst_pvalues, inst_output = get_peaks(name=options.name, states=states, DCS=exp_data,
                                                           distr=distr, merge=options.merge, exts=exp_data.exts,
                                                           pcutoff=options.pcutoff, debug=options.debug, p=options.par,
                                                           no_correction=options.no_correction,
                                                           deadzones=options.deadzones)

        output += inst_output
        pvalues += inst_pvalues
        ratios += inst_ratios
    
    res_output, res_pvalues, res_filter_pass = filter_by_pvalue_strand_lag(ratios, options.pcutoff, pvalues, output,
                                                                           options.no_correction, options.name,
                                                                           options.singlestrand)
    
    _output_BED(options.name, res_output, res_pvalues, res_filter_pass)
    _output_narrowPeak(options.name, res_output, res_pvalues, res_filter_pass)
    
    merge_output(bamfiles, dims, options, no_bw_files, chrom_sizes)


def main():
    options, bamfiles, genome, chrom_sizes, dims, inputs = handle_input()

    tracker = Tracker(options.name + '-setup.info', bamfiles, genome, chrom_sizes, dims, inputs, options, __version__)
    region_giver = RegionGiver(chrom_sizes, options.regions)
    m, exp_data, func_para, init_mu, init_alpha, distr = train_HMM(region_giver, options, bamfiles, genome,
                                                                   chrom_sizes, dims, inputs, tracker)
    
    run_HMM(region_giver, options, bamfiles, genome, chrom_sizes, dims, inputs, tracker, exp_data, m, distr)
    
    _write_info(tracker, options.report, func_para=func_para, init_mu=init_mu, init_alpha=init_alpha, m=m)



###################################################################################################
# Libraries
###################################################################################################

# Python
import warnings

warnings.filterwarnings("ignore")

# Internal
from ..Util import ErrorHandler


###################################################################################################
# Classes
###################################################################################################

class HMM:
    """
    Represent an HMM

    Authors: Eduardo G. Gusmao.

    Methods:

    load_hmm(input_file_name):
    Loads an HMM based on a .hmm file.
    """

    def __init__(self):
        """ 
        Initializes HMM.

        Variables:
        states -- Number of states of the HMM (integer).
        dim -- Number of dimensions of the HMM (integer).
        pi -- Initial state probabilities vector (list).
        A -- Transition matrix (list of lists).
        means -- Matrix containing the mean vector of each state (list of lists).
        covs -- List of covariance matrices (list of list of lists).
        """
        self.states = 0
        self.dim = 0
        self.pi = []
        self.A = []
        self.means = []
        self.covs = []

    def load_hmm(self, input_file_name):
        """ 
        Loads all objects of this class based on input_file_name.
        These objects are used to create a scikit-based HMM.

        Keyword arguments:
        input_file_name -- File location + name in HMM format.
        See manual for full description of such format.
        
        Return:
        None -- It loads variables of HMM with information from input_file_name
        """

        # Creating file
        input_file = open(input_file_name, "r")

        # Reading number of states
        hmm_states = int(input_file.readline().strip().split(" ")[1])
        input_file.readline()

        # Reading initial state probabilities
        pi = [float(e) for e in input_file.readline().strip().split(" ")]
        input_file.readline()

        # Reading transition matrix
        A = []
        for i in range(0, hmm_states): A.append([float(e) for e in input_file.readline().strip().split(" ")])
        input_file.readline()
        first_emis_line = input_file.readline()

        # Reading emission probabilities of multivariate HMM
        E = []
        for i in range(0, hmm_states):
            if (i == 0):
                Elist = first_emis_line.strip().split("#")
            else:
                Elist = input_file.readline().strip().split("#")
            E.append([[float(e) for e in Elist[0].strip().split(" ")], [float(e) for e in Elist[1].strip().split(" ")]])
        dim_nb = len(E[0][0])

        # Closing file
        input_file.close()

        # Preparing scikit structures
        means_vec = [e[0] for e in E]
        cov_vec = []
        for e in E:
            new_mat = []
            for i in range(0, dim_nb):
                new_vec = []
                for j in range(0, dim_nb):
                    new_vec.append(e[1][(dim_nb * i) + j])
                new_mat.append(new_vec)
            cov_vec.append(new_mat)

        # Create scikit HMM
        self.states = hmm_states
        self.dim = dim_nb
        self.pi = pi
        self.A = A
        self.means = means_vec
        self.covs = cov_vec

    def save_hmm(self, output_file_name, precision=6):
        """
        Save all objects of this class to the output_file_name.

        Keyword arguments:
        output_file_name -- File location + name in HMM format.

        Return:
        None -- It saves variables of HMM with information to output_file_name
        """
        output_file_name += ".hmm"
        with open(output_file_name, "w") as output_file:
            output_file.write("states " + str(self.states) + "\n")

            output_file.write("initial" + "\n")
            output_file.write(str(self.pi[0]))
            for e in self.pi[1:]:
                output_file.write(" " + str(e))

            output_file.write("\n" + "transitions" + "\n")
            for e in self.A:
                output_file.write(str(round(e[0], precision)))
                for v in e[1:]:
                    output_file.write(" " + str(round(v, precision)))
                output_file.write("\n")

            output_file.write("emissions" + "\n")
            for idx in range(self.states):
                output_file.write(str(round(self.means[idx][0], precision)))
                for e in self.means[idx][1:]:
                    output_file.write(" " + str(round(e, precision)))
                output_file.write(" # ")
                output_file.write(str(round(self.covs[idx][0], precision)))
                for e in self.covs[idx][1:]:
                    output_file.write(" " + str(round(e, precision)))
                output_file.write("\n")



###################################################################################################
# Libraries
###################################################################################################

# Python
from __future__ import print_function

# Internal
from rgt.GeneSet import GeneSet
from rgt.Util import OverlapType
from rgt.GenomicRegionSet import GenomicRegionSet

# External
from numpy import asarray, argsort, sum, arange, nonzero, minimum

###################################################################################################
# Functions
###################################################################################################


def ecdf(x):
    """
    Auxiliary function for multiple_test_correction
    """
    nobs = len(x)
    return arange(1,nobs+1)/float(nobs)


def multiple_test_correction(pvals, alpha=0.05, method='indep'):
    """ 
    p-value correction for false discovery rate.
   
    This covers Benjamini/Hochberg for independent or positively correlated and
    Benjamini/Yekutieli for general or negatively correlated tests. Both are
    available in the function multipletests, as method=`fdr_bh`, resp. `fdr_by`.

    If there is prior information on the fraction of true hypothesis, then alpha
    should be set to alpha * m/m_0 where m is the number of tests,
    given by the p-values, and m_0 is an estimate of the true hypothesis.
    (see Benjamini, Krieger and Yekuteli)

    The two-step method of Benjamini, Krieger and Yekutiel that estimates the number
    of false hypotheses will be available (soon).

    Method names can be abbreviated to first letter, 'i' or 'p' for fdr_bh and 'n' for
    fdr_by.

    Author: Josef Pktd, H Raja and Vincent Davis (scikits.statsmodels.sandbox.stats.multicomp)

    Keyword arguments:
    pvals -- List of p-values from the individual tests.
    alpha -- Error rate (float). (default 0.05)
    method -- {'indep', 'negcorr')
        
    Return:
    rejected -- List of booleans. True if a hypothesis is rejected, False otherwise.
    pvalue_corrected -- A list with the p-values adjusted for multiple hypothesis testing to limit FDR.
    """

    pvals = asarray(pvals)

    pvals_sortind = argsort(pvals)
    pvals_sorted = pvals[pvals_sortind]
    sortrevind = pvals_sortind.argsort()

    if method in ['i', 'indep', 'p', 'poscorr']:
        ecdffactor = ecdf(pvals_sorted)
    elif method in ['n', 'negcorr']:
        cm = sum(1./arange(1, len(pvals_sorted)+1))
        ecdffactor = ecdf(pvals_sorted) / cm
    else:
        raise ValueError('only indep and necorr implemented')
    reject = pvals_sorted < ecdffactor*alpha
    if reject.any():
        rejectmax = max(nonzero(reject)[0])
    else:
        rejectmax = 0
    reject[:rejectmax] = True

    pvals_corrected_raw = pvals_sorted / ecdffactor
    pvals_corrected = minimum.accumulate(pvals_corrected_raw[::-1])[::-1]
    pvals_corrected[pvals_corrected>1] = 1
    return reject[sortrevind], pvals_corrected[sortrevind]


def fisher_table(motif_name, regions, mpbs, gene_set=False, mpbs_set=False):
    """
    TODO

    Keyword arguments:
    motif_name -- TODO
    regions -- TODO
    mpbs -- TODO
    gene_set -- TODO
    mpbs_set -- TODO

    Return:
    a -- TODO
    b -- TODO
    gene_set -- TODO
    mpbs_set -- TODO
    """

    # Fetching motif
    mpbs_motif = GenomicRegionSet(name="mpbs_motif")
    for region in mpbs.sequences:
        if motif_name in region.name:
            mpbs_motif.add(region)

    # Performing intersections
    if len(mpbs_motif) > 0:
        # regions which are overlapping with mpbs_motif
        intersect_original = regions.intersect(mpbs_motif, mode=OverlapType.ORIGINAL, rm_duplicates=True)
        # regions which are not overlapping with regions from mpbs_motif
        subtract_overlap = regions.subtract(mpbs_motif, whole_region=True)

        # Fetching genes
        if gene_set:
            gene_set_res = GeneSet(motif_name)
            for genomic_region in intersect_original.sequences:
                if genomic_region.name:
                    gene_list = [e if e[0] != "." else e[1:] for e in genomic_region.name.split(":")]
                    for g in gene_list:
                        gene_set_res.genes.append(g)
            gene_set_res.genes = list(set(gene_set_res.genes))  # Keep only unique genes
        else:
            gene_set_res = None

        # Fetching mpbs
        if mpbs_set:
            mpbs_set_res = mpbs_motif.intersect(regions, mode=OverlapType.ORIGINAL, rm_duplicates=True)
        else:
            mpbs_set_res = None

        return len(intersect_original), len(subtract_overlap), gene_set_res, mpbs_set_res

    else:
        gene_set_res = GeneSet(motif_name) if gene_set else None
        mpbs_set_res = GenomicRegionSet(mpbs_motif.name) if mpbs_set else None

        return 0, len(regions), gene_set_res, mpbs_set_res


def get_fisher_dict(motif_names, regions, mpbs, gene_set=False, mpbs_set=False):
    """
    TODO

    Keyword arguments:
    motif_names -- TODO
    regions -- TODO
    mpbs -- TODO
    gene_set -- TODO
    mpbs_set -- TODO

    Return:
    res1_dict -- TODO
    res2_dict -- TODO
    geneset_dict -- TODO
    mpbs_dict -- TODO
    """

    # Calculating statistics for EV
    res1_dict = dict()
    res2_dict = dict()

    geneset_dict = dict()
    mpbs_dict = dict()

    for motif in motif_names:
        table = fisher_table(motif, regions, mpbs, gene_set=gene_set, mpbs_set=mpbs_set)

        # number of input regions intersecting mpbs regions
        res1_dict[motif] = table[0]
        # number of input regions NOT intersecting mpbs regions
        res2_dict[motif] = table[1]

        if gene_set:
            geneset_dict[motif] = table[2]

        # GenomicRegionSet of mpbs regions intersecting input regions
        if mpbs_set:
            mpbs_dict[motif] = table[3]

    # Return
    return res1_dict, res2_dict, geneset_dict, mpbs_dict



# Python Libraries
from __future__ import print_function
from __future__ import division
import itertools
import multiprocessing.pool
import matplotlib.ticker as mtick
from matplotlib_venn import venn3

# Local Libraries
# Distal Libraries
from shared_function import *

# Local test
dir = os.getcwd()

###########################################################################################
#                    Inersection test
###########################################################################################


class Intersect:
    def __init__(self, reference_path, query_path, mode_count, organism):
        self.rEM, self.qEM = ExperimentalMatrix(), ExperimentalMatrix()
        self.rEM.read(reference_path)
        self.rEM.remove_empty_regionset()
        self.references = self.rEM.get_regionsets()
        self.referencenames = self.rEM.get_regionsnames()
        self.qEM.read(query_path)
        self.qEM.remove_empty_regionset()
        self.query = self.qEM.get_regionsets()
        self.querynames = self.qEM.get_regionsnames()
        self.mode_count = mode_count
        self.organism = organism
        self.sbar = None
        self.pbar = None
        self.test_d = None
        # self.background = None

    def background(self, path=None):
        """Given a bed file as the background for analysis"""
        # bgbed = GenomicRegionSet(name="Background")
        # if path:
        #     bgbed.read_bed(path)
        #     # nq = []
        #     print("\tTrimming the queries by the given background: "+path)

        #     rlist = [ r.trim_by(background=bgbed) for r in self.references]
        #     self.references = rlist
        #     qlist = [ q.trim_by(background=bgbed) for q in self.query]
        #     self.query = qlist

        # self.background = bgbed

        if path:
            bg = GenomicRegionSet("background")
            bg.read_bed(path)
            self.background = bg
            for ty in self.groupedreference.keys():
                # self.background[ty] = bg
                rlist = [r.trim_by(background=bg) for r in self.groupedreference[ty]]
                self.groupedreference[ty] = rlist
                qlist = [q.trim_by(background=bg) for q in self.groupedquery[ty]]
                self.groupedquery[ty] = qlist

    def group_refque(self, groupby):
        self.groupedreference, self.groupedquery = group_refque(self.rEM, self.qEM,
                                                                groupby,
                                                                self.references,
                                                                self.query)
        remove_duplicates(self.groupedreference)
        remove_duplicates(self.groupedquery)

    def colors(self, colorby, definedinEM, ref_que="que"):
        """color_list is a Dict [query] : color """
        if ref_que == "que":
            self.color_list = color_groupded_region(self.qEM, self.groupedquery, colorby, definedinEM)
            if self.groupedquery.keys()[0] == "":
                self.color_tags = [n.name for n in self.groupedquery[""]]
            else:
                self.color_tags = gen_tags(self.qEM, colorby)
        elif ref_que == "ref":
            self.color_list = color_groupded_region(self.rEM, self.groupedreference, colorby, definedinEM)
            if self.groupedquery.keys()[0] == "":
                self.color_tags = [n.name for n in self.groupedquery[""]]
            else:
                self.color_tags = gen_tags(self.qEM, colorby)

    def colors_comb(self):
        """color_list is a list : color """

        if self.groupedquery.keys()[0] == "":
            self.color_tags = self.referencenames
        else:
            tags = []
            for t in [n.name for n in self.groupedreference.values()[0]]:
                nt = t.replace(self.groupedreference.keys()[0], "")
                nt = nt.replace("_", "")
                tags.append(nt)
            self.color_tags = tags
        self.color_list = plt.cm.Set1(numpy.linspace(0.1, 0.9, len(self.color_tags))).tolist()

    def extend_ref(self, percentage):
        """percentage must be positive value"""
        for ty in self.groupedreference:
            for r in self.groupedreference[ty]:
                r.extend(left=percentage, right=percentage, percentage=False)
                r.merge()

    def count_intersect(self, threshold, frequency=True):

        self.counts = OrderedDict()
        self.rlen, self.qlen = {}, {}
        self.nalist = []

        if frequency: self.frequency = OrderedDict()

        # if self.mode_count == "bp":
        #    print2(self.parameter, "\n{0}\t{1}\t{2}\t{3}\t{4}".format("Reference","Length(bp)", "Query", "Length(bp)", "Length of Intersection(bp)"))
        # elif self.mode_count == "count":
        #    print2(self.parameter, "\n{0}\t{1}\t{2}\t{3}\t{4}".format("Reference","sequence_number", "Query", "sequence_number", "Number of Intersection"))

        for ty in self.groupedreference.keys():
            self.counts[ty] = OrderedDict()
            self.rlen[ty], self.qlen[ty] = OrderedDict(), OrderedDict()
            if frequency: self.frequency[ty] = OrderedDict()

            for r in self.groupedreference[ty]:
                if r.total_coverage() == 0 and len(r) > 0:
                    self.nalist.append(r.name)
                    continue
                else:
                    self.counts[ty][r.name] = OrderedDict()
                    if self.mode_count == "bp":
                        rlen = r.total_coverage()
                    elif self.mode_count == "count":
                        rlen = len(r)
                    self.rlen[ty][r.name] = rlen

                    mp_input = []
                    for q in self.groupedquery[ty]:
                        if r.name == q.name:
                            continue
                        else:
                            mp_input.append([q, self.nalist, self.mode_count, self.qlen, threshold,
                                             self.counts, frequency, self.frequency, ty, r])
                    # q, nalist, mode_count, qlen_dict, threshold, counts, frequency, self_frequency, ty, r
                    pool = multiprocessing.Pool(processes=multiprocessing.cpu_count()-1)
                    mp_output = pool.map(mp_count_intersect, mp_input)
                    pool.close()
                    pool.join()

                    # qname, nalist, qlen_dict[ty][q.name], counts[ty][r.name][q.name], self_frequency[ty][q.name].append(c[2])
                    for output in mp_output:
                        if output[1]:
                            self.nalist.append(output[1])
                        else:
                            self.qlen[ty][output[0]] = output[2]
                            self.counts[ty][r.name][output[0]] = output[3]
                            # print(r.name)
                            # print(output[0])
                            # print(output[3])
                            try:
                                self.frequency[ty][output[0]][r.name] = output[3][2]
                            except:
                                self.frequency[ty][output[0]] = {}
                                self.frequency[ty][output[0]][r.name] = output[3][2]
                                # print2(self.parameter, "{0}\t{1}\t{2}\t{3}\t{4}".format(r.name,rlen, q.name, qlen, c[2]))
                                # print(self.nalist)
                                # print(self.qlen)
                                # print(self.counts)
                                # print(self.frequency)
                                # sys.stdout.flush()

    def barplot(self, logt=False, percentage=False):
        f, axs = plt.subplots(len(self.counts.keys()), 1)
        f.subplots_adjust(left=0.3)
        self.xtickrotation, self.xtickalign = 0, "center"
        # if len(axs) == 1: axs = [axs]
        try:
            axs = axs.reshape(-1)
        except:
            axs = [axs]

        for ai, ax in enumerate(axs):
            if logt:
                ax.set_yscale('log')
                plus = 1
            else:
                ax.locator_params(axis='y', nbins=4)
                plus = 0
                ax.ticklabel_format(style='sci', axis='y', scilimits=(0, 0))
            ax.set_title(self.counts.keys()[ai], y=1)

            r_label = []
            for ind_r, r in enumerate(self.counts.values()[ai].keys()):
                for l in self.references:
                    if l.name == r:
                        lr = len(l)

                if len(axs) == 1:
                    r_label.append(r)
                else:
                    try:
                        r_label.append(self.rEM.get_type(r, "factor"))
                    except:
                        r_label.append(r)
                if len(r_label[-1]) > 15 or len(self.counts.values()[ai][r].keys()) * len(
                        self.counts.values()[ai].keys()) > 8:
                    self.xtickrotation, self.xtickalign = 50, "right"
                width = 0.8 / (len(self.counts.values()[ai][r].keys()) + 1)  # Plus one background
                for ind_q, q in enumerate(self.counts.values()[ai][r].keys()):
                    x = ind_r + ind_q * width + 0.1
                    if percentage:
                        # print(lr)
                        y = 100 * (self.counts.values()[ai][r][q][2] + plus) / lr
                    else:
                        y = self.counts.values()[ai][r][q][2] + plus  # intersect number

                    ax.bar(x, y, width=width, color=self.color_list[q], edgecolor="none", align='edge', log=logt)

            ax.yaxis.tick_left()
            ax.set_xticks([i + 0.5 - 0.5 * width for i in range(len(r_label))])
            ax.set_xticklabels(r_label, fontsize=9, rotation=self.xtickrotation, ha=self.xtickalign)
            ax.tick_params(axis='x', which='both', top='off', bottom='off', labelbottom='on')
            ax.set_xlim([0, len(self.counts.values()[ai].keys()) - 0.1])

            ax.legend(self.color_tags, loc='center left', handlelength=1, handletextpad=1,
                      columnspacing=2, borderaxespad=0., prop={'size': 10}, bbox_to_anchor=(1.05, 0.5))
            for spine in ['top', 'right']:  # 'left', 'bottom'
                ax.spines[spine].set_visible(False)
        if self.mode_count == "bp":
            f.text(-0.025, 0.5, "Intersected regions (bp)", rotation="vertical", va="center")
        elif self.mode_count == "count":
            if percentage:
                fmt = '%.0f%%'  # Format you want the ticks, e.g. '40%'
                yticks = mtick.FormatStrFormatter(fmt)
                ax.yaxis.set_major_formatter(yticks)

                f.text(-0.025, 0.5, "Intersected percantage", rotation="vertical", va="center")
            else:
                f.text(-0.025, 0.5, "Intersected regions number", rotation="vertical", va="center")

        # f.tight_layout(pad=0.5, h_pad=None, w_pad=0.5)
        f.tight_layout()
        self.bar = f

    def stackedbar(self):
        f, axs = plt.subplots(len(self.counts.keys()), 1)
        f.subplots_adjust(left=0.3)
        # if len(axs) == 1: axs = [axs]
        try:
            axs = axs.reshape(-1)
        except:
            axs = [axs]

        for ai, ax in enumerate(axs):
            ax.ticklabel_format(style='sci', axis='y', scilimits=(0, 0))
            ax.locator_params(axis='y', nbins=2)
            ax.set_title(self.counts.keys()[ai], y=1)
            r_label = []

            for ind_r, r in enumerate(self.counts.values()[ai].keys()):
                if len(axs) == 1:
                    r_label.append(r)
                else:
                    try:
                        r_label.append(self.rEM.get_type(r, "factor"))
                    except:
                        r_label.append(r)
                width = 0.6
                bottom = 0
                reverse_q = self.counts.values()[ai][r].keys()[::-1]
                # ql = len(reverse_q) - 1
                rc = self.color_tags[::-1]
                for ind_q, q in enumerate(reverse_q):
                    x = ind_r
                    y = self.counts.values()[ai][r][q][2]  # intersect number
                    ax.bar(x, y, width=width, bottom=bottom, color=self.color_list[q],
                           edgecolor="none", align='center', label=rc[ind_q])
                    bottom = bottom + y
            ax.yaxis.tick_left()
            ax.set_xticks(range(len(r_label)))
            ax.set_xticklabels(r_label, fontsize=9, rotation=self.xtickrotation, ha=self.xtickalign)
            ax.tick_params(axis='x', which='both', top='off', bottom='off', labelbottom='on')
            ax.set_xlim([-0.5, ind_r + 0.5])

            # handles, labels = ax.get_legend_handles_labels()
            # nhandles = []
            # for t in self.color_tags:
            #    nhandles.append( handles[labels.index(t)] )
            # print(self.color_tags)
            handles, labels = ax.get_legend_handles_labels()
            by_label = OrderedDict(reversed(zip(labels, handles)))
            ax.legend(by_label.values(), by_label.keys(), loc='center left', handlelength=1, handletextpad=1,
                      columnspacing=2, borderaxespad=0., prop={'size': 10}, bbox_to_anchor=(1.05, 0.5))
            for spine in ['top', 'right']:  # 'left', 'bottom'
                ax.spines[spine].set_visible(False)
        if self.mode_count == "bp":
            f.text(-0.025, 0.5, "Intersected regions (bp)", rotation="vertical", va="center")
        elif self.mode_count == "count":
            f.text(-0.025, 0.5, "Intersected regions number", rotation="vertical", va="center")

        # f.tight_layout(pad=0.5, h_pad=None, w_pad=0.5)
        f.tight_layout()
        self.sbar = f

    def percentagebar(self):
        self.color_list["No intersection"] = "0.7"
        f, axs = plt.subplots(len(self.counts.keys()), 1)
        f.subplots_adjust(left=0.3)
        # if len(axs) == 1: axs = [axs]
        try:
            axs = axs.reshape(-1)
        except:
            axs = [axs]
        self.percentage = []

        for ai, ax in enumerate(axs):
            # ax.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
            ax.locator_params(axis='y', nbins=2)
            ax.set_title(self.counts.keys()[ai], y=1.05)
            r_label = []
            self.percentage.append({})
            for ind_r, r in enumerate(self.counts.values()[ai].keys()):
                if len(axs) == 1:
                    r_label.append(r)
                else:
                    try:
                        r_label.append(self.rEM.get_type(r, "factor"))
                    except:
                        r_label.append(r)
                width = 0.6
                bottom = 0
                if self.mode_count == "bp":
                    sumlength = self.rEM.objectsDict[r].total_coverage()
                elif self.mode_count == "count":
                    sumlength = len(self.rEM.objectsDict[r])
                self.percentage[ai][r] = {}

                if sumlength == 0:
                    for ind_q, q in enumerate(self.counts.values()[ai][r].keys()):
                        self.percentage[ai][r][q] = "ref is empty"
                else:
                    for ind_q, q in enumerate(self.counts.values()[ai][r].keys()):
                        x = ind_r
                        y = 100 * float(self.counts.values()[ai][r][q][2]) / sumlength  # percentage
                        ax.bar(x, y, width=width, bottom=bottom,
                               color=self.color_list[q], edgecolor="none", align='center')
                        bottom = bottom + y
                        self.percentage[ai][r][q] = y

                ax.bar(x, 100 - bottom, width=width, bottom=bottom,
                       color=self.color_list["No intersection"], align='center')

            ax.yaxis.tick_left()

            ax.set_xticks(range(len(r_label)))
            ax.set_xticklabels(r_label, fontsize=9, rotation=self.xtickrotation, ha=self.xtickalign)
            ax.tick_params(axis='x', which='both', top='off', bottom='off', labelbottom='on')
            ax.set_xlim([-0.5, ind_r + 0.5])
            ax.set_ylim([0, 100])

            legend_labels = self.color_tags + ["No intersection"]
            ax.legend(legend_labels, loc='center left', handlelength=1, handletextpad=1,
                      columnspacing=2, borderaxespad=0., prop={'size': 10}, bbox_to_anchor=(1.05, 0.5))
            for spine in ['top', 'right']:  # 'left', 'bottom'
                ax.spines[spine].set_visible(False)
        f.text(-0.025, 0.5, "Proportion of intersected regions (%)", rotation="vertical", va="center")

        # f.tight_layout(pad=0.5, h_pad=None, w_pad=0.5)
        f.tight_layout()
        self.pbar = f

    def gen_html(self, directory, title, align, args):
        # fp = os.path.join(dir,outputname,title)
        # link_d = {title:"intersection.html"}
        dir_name = os.path.basename(directory)
        # check_dir(directory)
        html_header = "Intersection Test: " + dir_name + "/" + title
        link_d = OrderedDict()
        link_d["Intersection test"] = "index.html"
        link_d["Parameters"] = "parameters.html"

        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")

        html.add_figure("intersection_bar.png", align="center")
        html.add_figure("intersection_barp.png", align="center")
        if self.sbar: html.add_figure("intersection_stackedbar.png", align="center")

        header_list = ["#",
                       "Reference<br>name",
                       "Query<br>name",
                       "Reference<br>number",
                       "Query<br>number",
                       "Intersect.",
                       "Proportion<br>of Reference"]
        statistic_table = [["Reference_name", "Query_name", "Reference_number", "Query_number",
                            "Intersect.", "Proportion_of_Reference"]]
        if self.test_d:
            header_list += ["Average<br>intersect.", "Chi-square<br>statistic",
                            "Positive<br>Association<br>p-value", "Negative<br>Association<br>p-value"]
            statistic_table[0] += ["Average_intersect.", "Chi-square_statistic",
                                   "Positive_Association_p-value", "Negative_Association_p-value"]
        else:
            pass

        type_list = 'ssssssssssssssss'
        col_size_list = [10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10]

        for ind_ty, ty in enumerate(self.counts.keys()):
            html.add_heading(ty, size=4, bold=False)
            data_table = []
            c = 0
            for ind_r, r in enumerate(self.counts[ty]):
                for ind_q, q in enumerate(self.counts[ty][r]):
                    if r == q: continue
                    c += 1
                    pt = self.counts[ty][r][q][2] / self.rlen[ty][r]
                    intern = self.counts[ty][r][q][2]
                    if self.test_d:
                        aveinter = self.test_d[ty][r][q][0]
                        chisqua = value2str(self.test_d[ty][r][q][1])
                        pv = self.test_d[ty][r][q][2]
                        if isinstance(pv, str):
                            data_table.append([str(c), r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]),
                                               str(intern), "{:.2f}%".format(100 * pt),
                                               aveinter, chisqua, pv, "-"])
                        else:
                            npv = 1 - pv
                            if pv < 0.05:
                                if intern > aveinter:
                                    data_table.append([str(c), r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]),
                                                       str(intern), "{:.2f}%".format(100 * pt),
                                                       value2str(aveinter), chisqua,
                                                       "<font color=\"red\">" + value2str(pv) + "</font>",
                                                       value2str(npv)])
                                    statistic_table.append(
                                        [r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]), str(intern),
                                         "{:.2f}%".format(100 * pt),
                                         value2str(aveinter), chisqua, value2str(pv), value2str(npv)])
                                else:
                                    data_table.append([str(c), r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]),
                                                       str(intern), "{:.2f}%".format(100 * pt),
                                                       value2str(aveinter), chisqua, value2str(npv),
                                                       "<font color=\"red\">" + value2str(pv) + "</font>"])
                                    statistic_table.append(
                                        [r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]), str(intern),
                                         "{:.2f}%".format(100 * pt),
                                         value2str(aveinter), chisqua, value2str(npv), value2str(pv)])
                            elif self.test_d[ty][r][q][2] >= 0.05:
                                if intern > aveinter:
                                    data_table.append([str(c), r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]),
                                                       str(intern), "{:.2f}%".format(100 * pt),
                                                       value2str(aveinter), chisqua, value2str(pv), value2str(npv)])
                                    statistic_table.append(
                                        [r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]), str(intern),
                                         "{:.2f}%".format(100 * pt),
                                         value2str(aveinter), chisqua, value2str(pv), value2str(npv)])
                                else:
                                    data_table.append([str(c), r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]),
                                                       str(intern), "{:.2f}%".format(100 * pt),
                                                       value2str(aveinter), chisqua, value2str(npv), value2str(pv)])
                                    statistic_table.append(
                                        [r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]), str(intern),
                                         "{:.2f}%".format(100 * pt),
                                         value2str(aveinter), chisqua, value2str(npv), value2str(pv)])
                    else:
                        data_table.append([str(c), r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]),
                                           str(intern), "{:.2f}%".format(100 * pt)])
                        statistic_table.append([r, q, str(self.rlen[ty][r]), str(self.qlen[ty][q]), str(intern),
                                                "{:.2f}%".format(100 * pt)])

            html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align, sortable=True)
            output_array(statistic_table, directory=directory, folder=title, filename="statistics" + ty + ".txt")

        html.add_heading("Assumptions and hypothesis")
        list_ex = ['Positive association is defined by: True intersection number > Averaged random intersection.',
                   'Negative association is defined by: True intersection number < Averaged random intersection.']

        self.nalist = set(self.nalist)
        if len(self.nalist) > 0:
            list_ex.append(
                'The following region sets contain zero-length regions which cause error in intersection calculation, please check it:<br>' +
                '<font color=\"red\">' + ', '.join([s for s in self.nalist]) + '</font>')
        if self.test_d:
            list_ex.append('Randomly permutation for ' + str(self.test_time) + ' times.')
        html.add_list(list_ex)
        html.add_fixed_rank_sortable()
        html.write(os.path.join(directory, title, "index.html"))

        # Parameters
        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")
        header_list = ["Description", "Argument", "Value"]
        data_table = [["Reference", "-r", args.r],
                      ["Query", "-q", args.q],
                      ["Output directory", "-o", os.path.basename(args.o)],
                      ["Experiment title", "-t", args.t],
                      # ["Grouping tag", "-g", args.g],
                      # ["Coloring tag", "-c", args.c],
                      # ["Background", "-bg", args.bg],
                      ["Organism", "-organism", args.organism]]

        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align, cell_align="left")
        html.add_free_content([
                                  '<a href="reference_experimental_matrix.txt" style="margin-left:100">See reference experimental matrix</a>'])
        html.add_free_content(
            ['<a href="query_experimental_matrix.txt" style="margin-left:100">See query experimental matrix</a>'])
        html.add_free_content(['<a href="parameters.txt" style="margin-left:100">See details</a>'])
        html.write(os.path.join(directory, title, "parameters.html"))

    def gen_html_comb(self, directory, title, align, args):
        dir_name = os.path.basename(directory)
        # check_dir(directory)
        html_header = "Combinatorial Test: " + dir_name + "/" + title
        link_d = OrderedDict()
        link_d["Combinatorial test"] = "index.html"
        link_d["Parameters"] = "parameters.html"

        html = Html(name=html_header, links_dict=link_d, fig_dir=os.path.join(directory, "style"),
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")

        # html.create_header()
        # html.add_heading(title)

        if self.sbar: html.add_figure("intersection_stackedbar.png", align="center")

        # html.add_free_content(['<p style=\"margin-left: '+str(align+150)+'">'+ '** </p>'])

        for ind_ty, ty in enumerate(self.groupedreference.keys()):
            html.add_heading(ty, size=4, bold=False)

            data_table = []
            header_list = ["Query<br>name", "Query<br>number", "Frequencies"]
            type_list = 'sssss'
            col_size_list = [10, 10, 30, 10]
            # print(self.frequency[ty].keys())
            sys.stdout.flush()

            for ind_q, q in enumerate(self.frequency[ty].keys()):
                html.add_figure("venn_" + ty + "_" + q + ".png", align="center", width="600")
                data_table.append([q, str(self.qlen[ty][q]),
                                   ",".join([str(v).rjust(7, " ") for v in self.frequency[ty][q].values()])])

            html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align)

            header_list = ["Query 1", "Query 2", "Chi square", "p value"]
            data_table = []
            n = len(self.frequency[ty].keys())
            for q in itertools.combinations(range(n), 2):
                # print(q)
                q1 = self.frequency[ty].keys()[q[0]]
                q2 = self.frequency[ty].keys()[q[1]]

                sumqf = [x + y for x, y in zip(self.frequency[ty][q1].values(), self.frequency[ty][q2].values())]
                nonzero = [i for i, e in enumerate(sumqf) if e != 0]
                qf1 = [self.frequency[ty][q1].values()[i] for i in nonzero]
                qf2 = [self.frequency[ty][q2].values()[i] for i in nonzero]

                chisq, p, dof, expected = stats.chi2_contingency([qf1, qf2])
                if p < 0.05:
                    data_table.append([q1, q2, value2str(chisq), "<font color=\"red\">" + value2str(p) + "</font>"])
                else:
                    data_table.append([q1, q2, value2str(chisq), value2str(p)])

            if len(data_table):
                html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align)
            else:
                html.add_free_content(["No overlapping regions found."])
        """
        header_list = ["Order"] + self.orig_refs
        type_list = 'sssss' * len(self.referencenames)
        col_size_list = [10,10,10,10] * len(self.referencenames)
        data_table = []
        print(self.referencenames)
        print(self.comb_ref_infor.keys())
        for i in range(len(self.referencenames)):
            data_table.append([i+1] + self.comb_ref_infor.values()[i])
        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align = align)
        """

        html.write(os.path.join(dir_name, title, "index.html"))

        # Parameters
        html = Html(name=html_header, links_dict=link_d,
                    fig_rpath="../style", RGT_header=False, other_logo="viz", homepage="../index.html")
        header_list = ["Description", "Argument", "Value"]
        data_table = [["Reference", "-r", args.r],
                      ["Query", "-q", args.q],
                      ["Output directory", "-o", os.path.basename(args.o)],
                      ["Experiment title", "-t", args.t],
                      # ["Grouping tag", "-g", args.g],
                      # ["Coloring tag", "-c", args.c],
                      # ["Background", "-bg", args.bg],
                      ["Organism", "-organism", args.organism]]

        html.add_zebra_table(header_list, col_size_list, type_list, data_table, align=align, cell_align="left")
        html.add_free_content([
                                  '<a href="reference_experimental_matrix.txt" style="margin-left:100">See reference experimental matrix</a>'])
        html.add_free_content(
            ['<a href="query_experimental_matrix.txt" style="margin-left:100">See query experimental matrix</a>'])
        html.add_free_content(['<a href="parameters.txt" style="margin-left:100">See details</a>'])
        html.write(os.path.join(directory, title, "parameters.html"))

    def posi2region(self, regions, p):
        # all = range(len(regions))
        new_r = GenomicRegionSet(name="")
        for r in p:
            new_r.combine(regions[r])
        return new_r

    def posi2set(self, regions, p):
        alln = range(len(regions))
        inter_r = copy.deepcopy(regions[p[0]])

        for i in alln:
            # print("inter_r: "+inter_r.name)
            if i in p[1:]:
                inter_r = inter_r.intersect(regions[i], mode=OverlapType.OVERLAP)
            elif i == p[0]:
                pass
            else:
                inter_r = inter_r.subtract(regions[i], whole_region=False)
        # print("inter_r: "+inter_r.name)
        return inter_r

    def combinatorial(self, background=None):
        def p2sign(plist, length):
            output = ["-"] * length
            for p in plist:
                output[p] = "+"
            return output

        new_refsp = OrderedDict()
        new_refs = OrderedDict()
        ref_names = []
        self.comb_ref_infor = {}

        for ty in self.groupedreference.keys():
            n = len(self.groupedreference[ty])
            new_refs[ty] = []
            new_refsp[ty] = []

            for i in range(1, n):
                new_refsp[ty].append(itertools.combinations(range(n), i))
            for posi in new_refsp[ty]:
                # print(posi)
                posi = [list(i) for i in posi]

                for p in posi:
                    # print("   " + str(p))
                    pr = self.posi2set(self.groupedreference[ty], p)
                    new_refs[ty].append(pr)
                    ref_names.append(pr.name)
                    self.comb_ref_infor[pr.name] = p2sign(p, n)
            all_int = self.posi2set(self.groupedreference[ty], range(n))
            new_refs[ty].append(all_int)
            ref_names.append(all_int.name)
            self.comb_ref_infor[all_int.name] = p2sign(range(n), n)
            """
            # Background
            unions = GenomicRegionSet(name="")
            for r in self.groupedreference[ty]:
                unions.combine(r)
            unions.name = " + ".join([r.name for r in self.groupedreference[ty]])

            nonset = self.backgroung.subtract(unions)
            nonset.name = "!("+"+".join([r.name for r in self.groupedreference[ty]]) + ")"
            new_refs[ty].append(nonset)
            ref_names.append(nonset.name)
            """
        # self.comb_reference = new_refs
        self.groupedreference = copy.deepcopy(new_refs)
        self.orig_refs = copy.deepcopy(self.referencenames)
        self.referencenames = list(set(ref_names))

    def combine_regions(self, background=None):
        new_refsp = OrderedDict()
        new_refs = OrderedDict()
        ref_names = []

        for ty in self.groupedreference.keys():
            n = len(self.groupedreference[ty])
            new_refs[ty] = []
            new_refsp[ty] = []
            for i in range(1, n):
                new_refsp[ty].append(itertools.combinations(range(n), i))
            for posi in new_refsp[ty]:
                posi = [list(i) for i in posi]
                for p in posi:
                    # print("   " + str(p))
                    pr = self.posi2region(self.groupedreference[ty], p)
                    new_refs[ty].append(pr)
                    ref_names.append(pr.name)

    def comb_stacked_plot(self):
        self.xtickrotation, self.xtickalign = 0, "center"
        f, axs = plt.subplots(1, len(self.frequency.keys()), sharey=True)
        f.subplots_adjust(left=0.3)
        # f.set_size_inches(18.5,15)
        # if len(axs) == 1: axs = [axs]
        try:
            axs = axs.reshape(-1)
        except:
            axs = [axs]

        for ai, ax in enumerate(axs):
            # ax.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
            ty = self.frequency.keys()[ai]
            ax.locator_params(axis='y', nbins=4)
            ax.set_title(self.frequency.keys()[ai], y=1)
            r_label = []
            q_label = []
            legends = []

            for ind_q, q in enumerate(self.frequency[ty].keys()):
                if len(axs) == 1:
                    q_label.append(q)
                else:
                    try:
                        q_label.append(self.qEM.get_type(q, "factor"))
                    except:
                        q_label.append(q)
                width = 0.6
                bottom = 0
                summ = sum(self.frequency[ty][q].values())
                for ind_r, r in enumerate(self.referencenames):
                    # for ind_r, rc in enumerate(self.frequency[ty][q]):
                    rc = self.frequency[ty][q][r]
                    if ind_q == 0:
                        # r = self.groupedreference[ty][ind_r].name
                        r_label.append(r)
                    x = ind_q
                    y = rc / summ  # intersect number
                    bar = ax.bar(x, y, width=width, bottom=bottom, color=self.color_list[ind_r], align='center')
                    bottom = bottom + y
                    if ind_q == 0: legends.append(bar)
            ax.yaxis.tick_left()
            ax.set_xticks(range(len(q_label)))
            ax.set_xticklabels(q_label, fontsize=9, rotation=self.xtickrotation, ha=self.xtickalign)
            ax.tick_params(axis='x', which='both', top='off', bottom='off', labelbottom='on')
            ax.set_xlim([-0.5, len(q_label) + 0.5])
            ax.set_ylim([0, 1])

            legends.reverse()
            r_label.reverse()

            for spine in ['top', 'right']:  # 'left', 'bottom'
                ax.spines[spine].set_visible(False)

        legend_name = reversed(self.color_tags)
        axs[-1].legend(legends, legend_name, loc='upper left', handlelength=1, handletextpad=1,
                       columnspacing=2, borderaxespad=0., prop={'size': 10}, bbox_to_anchor=(1.05, 1))
        if self.mode_count == "bp":
            f.text(-0.025, 0.5, "Intersected regions (bp)", rotation="vertical", va="center")
        elif self.mode_count == "count":
            f.text(-0.025, 0.5, "Percentage", rotation="vertical", va="center")

        f.tight_layout(pad=0.5, h_pad=None, w_pad=0.5)
        self.sbar = f

    def comb_venn(self, directory):
        if len(self.references) == 2:
            print(2)
        elif len(self.references) == 3:
            for ind_ty, ty in enumerate(self.groupedreference.keys()):
                for q in self.query:
                    plt.figure(figsize=(6, 4))
                    plt.title("Venn Diagram: " + q.name)
                    freq = []
                    for r in self.groupedreference[ty]:
                        freq.append(self.frequency[ty][q.name][r.name])

                    # print([r.name for r in self.groupedreference[ty]])
                    self.venn = venn3(subsets=[freq[i] for i in [0, 1, 3, 2, 4, 5, 6]],
                                      set_labels=[n.name for n in self.references])
                    plt.annotate(str(len(q) - sum(freq)), xy=(0.1, 0.1), xytext=(-120, -120),
                                 ha='left', textcoords='offset points',
                                 bbox=dict(boxstyle='round,pad=0.5', fc='gray', alpha=0.1))
                    plt.savefig(os.path.join(directory, "venn_" + ty + "_" + q.name + ".png"))
                    plt.savefig(os.path.join(directory, "venn_" + ty + "_" + q.name + ".pdf"), format='pdf')
        else:
            print("*** For plotting Venn diagram, the number of references must be 2 or 3.")

    def stest(self, repeat, threshold, mp):

        print("\n\tIntersection random subsampling test:\n    Repeat " + str(repeat) + " times\n")
        self.test_time = repeat
        self.test_d = {}
        plist = OrderedDict()

        for ty in self.groupedreference.keys():
            self.test_d[ty] = {}
            plist[ty] = OrderedDict()
            for r in self.groupedreference[ty]:
                if r.name in self.nalist: continue
                print("\t" + r.name)
                self.test_d[ty][r.name] = {}
                plist[ty][r.name] = OrderedDict()
                print("\t.", end="")
                sys.stdout.flush()
                for q in self.groupedquery[ty]:
                    if r.name == q.name:
                        continue
                    else:
                        print(".", end="")
                        sys.stdout.flush()
                        if q.name in self.nalist: continue
                        # True intersection
                        obs = self.counts[ty][r.name][q.name]
                        qn = q.name
                        if obs[2] == 0:
                            aveinter, chisq, p = "NA", "NA", "1"
                        else:
                            com = q.combine(r, change_name=False, output=True)
                            # Randomization
                            d = []

                            inp = [com, self.rlen[ty][r.name], self.mode_count, threshold]
                            mp_input = [inp for i in range(repeat)]

                            pool = multiprocessing.Pool(processes=mp)
                            mp_output = pool.map(mp_count_intersets, mp_input)
                            pool.close()
                            pool.join()

                            # for i in range(repeat):
                            #    random_r,random_q = com.random_split(size=self.rlen[ty][r.name])
                            #    d.append(random_r.intersect_count(random_q, mode_count=self.mode_count, threshold=threshold))
                            # d.append(count_intersect(random_r, random_q, mode_count=self.mode_count, threshold=threshold))
                            da = numpy.array(mp_output)

                            exp_m = numpy.mean(da, axis=0)
                            # print(exp_m)
                            # print(obs)
                            chisq, p, dof, expected = stats.chi2_contingency([exp_m, obs])
                            aveinter = exp_m[2]

                        plist[ty][r.name][qn] = p
                        self.test_d[ty][r.name][qn] = [aveinter, chisq, p]
                print()

            multiple_correction(plist)

            # c_p = 0
            for r in self.test_d[ty].keys():
                if r in self.nalist: continue
                for q in self.test_d[ty][r].keys():
                    self.test_d[ty][r][q][2] = plist[ty][r][q]
